'****************************************************************************
'* PROG REMA6.BAS                           REMA for magnetometer JR-6/JR-6A
'* ver. 1.6  21-Nov-06
'* ver. 1.7  08-Feb-07 receive with cmd ?                                   
'****************************************************************************

DECLARE SUB Clearbuff ()
DECLARE SUB Pause (T!)
DECLARE SUB Uhel (x1, y1, z1, x2, y2, z2, Alfa)
DECLARE FUNCTION ENG$ (a!, masize%)

version:
version$ = "ver. 1.7"                ' !!!!!!!!!!
simul = -0                           ' !!! manual holder only !!!
												 ' !!! simul by itself, see simuldata

OPTION BASE 1

DIM doc(1 TO 17) AS STRING * 50
DIM doc1(7 TO 8) AS STRING * 50
DIM doc2 AS STRING * 50, doc7 AS STRING * 50, doc7p AS STRING * 50

DIM T(12), Top(12)
DIM R(3, 3), R1(3, 3), R2(3, 3), a(3, 3), b(3, 3), C(3, 3), P(3), d(3)
DIM a1rep(10), a2rep(10)

doc(1) = "F1 - MEASURING ROUTINE"
doc(2) = "F2 - Serial Communication Port Number [ 1 / 2 ]"
doc2 = "F2 - Tectonic Correction"
doc(3) = "F3 - Orientation Parameters"
doc(4) = "F4 - Specimen Volume"
doc(5) = "F5 - CALIBRATION ROUTINE"
doc(6) = "F6 - Calibration Standard Magnetization and Volume"
doc(7) = "F7 - Holder Type: 0-Auto  1-SemiAuto  2-Manual"
doc7 = "   - Specimen Type: 0-Cylinder  1-Cube"
doc(8) = "F8 - Speed of Rotation: 0-High  1-Low"
doc(9) = "F9 - CORRECTION FOR HOLDER"
doc(10) = "F10- Exit Program"
doc(11) = "     Gain and Phase for High Speed"
doc(12) = "     Gain and Phase for Low Speed"
doc(13) = "     Standard Holder Remanent Values"
doc(14) = "     Auto Holder Remanent Values - P1"
doc(15) = "     Auto Holder Remanent Values - P2"
doc(16) = "     Auto Holder Remanent Values - P3"


doc1(7) = "F7 - Holder Type [ Auto / SemiAuto / Manual ]"
doc7p = "   - Specimen Type [ Cylinder / Cube ]"
doc1(8) = "F8 - Speed of Rotation [ High / Low ]"
						 
'------------------------------------------------------------------

DIM bfs(2, 2) AS STRING * 1
TYPE geol1                             ' 1st record in .GED file
	num AS STRING * 4              ' Number of specimens in file+2
	namloc AS STRING * 16          ' literary name of locality
	long AS STRING * 7             ' geographical longitude of locality
	lat AS STRING * 7              ' geographical latitude of locality
	rock AS STRING * 4             ' rock type
	strat AS STRING * 4            ' stratigraphy
	lito AS STRING * 4             ' lithostratigraphy
	regi AS STRING * 4             ' regional geological position
	pr1 AS STRING * 3
	pr2 AS STRING * 3              ' pr1 to pr4 orientation parameters
	pr3 AS STRING * 3
	pr4 AS STRING * 3
	crlf AS STRING * 2
END TYPE
DIM geol1 AS geol1

TYPE geol                              ' any record in .GED but not 1st
	nam AS STRING * 12             ' name of specimen
	azim AS STRING * 4             ' first orientation angle
	plun AS STRING * 4             ' second orientation angle
	bfs11 AS STRING * 1            ' code of first meso foliation
	bfs21 AS STRING * 1            ' code of first meso lineation
	dum AS STRING * 2
	dfoli1 AS STRING * 4           ' azimuth of the first foliation
	ifoli1 AS STRING * 4           ' dip of the first foliation
	dline1 AS STRING * 4           ' trend of the first lineation
	iline1 AS STRING * 4           ' plunge of the first lineation
	bfs12 AS STRING * 1            ' code of second meso foliation
	bfs22 AS STRING * 1            ' code of second meso linetion
	dummy AS STRING * 2
	dfoli2 AS STRING * 4           ' azimuth of the  second foliation
	ifoli2 AS STRING * 4           ' dip of the second foliation
	dline2 AS STRING * 4           ' trend of the second lineation
	iline2 AS STRING * 4           ' plunge of the second lineation
	crlf AS STRING * 4
END TYPE
DIM geol AS geol

TYPE hetype                            ' 1st (header) record, in .ran file
	nrec AS INTEGER
	nameloc AS STRING * 16
	longi AS STRING * 7
	lati AS STRING * 7
	rockt AS STRING * 4
	strati AS STRING * 4
	litos AS STRING * 4
	regio AS STRING * 4
	pp1 AS STRING * 3
	pp2  AS STRING * 3
	pp3 AS STRING * 3
	pp4  AS STRING * 3
	vypl AS STRING * 4
END TYPE
DIM he AS hetype

TYPE datype                            ' data record in .ran file
	names AS STRING * 12
	gk AS SINGLE
	gn AS SINGLE
	kalg(1 TO 6) AS SINGLE
	bfs11 AS STRING * 1
	bfs21 AS STRING * 1
	irho1 AS INTEGER
	iamu1 AS INTEGER
	itau1 AS INTEGER
	isig1 AS INTEGER
	bfs12 AS STRING * 1
	bfs22 AS STRING * 1
	irho2 AS INTEGER
	iamu2 AS INTEGER
	itau2 AS INTEGER
	isig2 AS INTEGER
END TYPE
DIM da AS datype
'---------------------------------
 s& = STACK - 10000&
 STACK s&

 GOSUB keysoff
 Tectcorr = 1                     'default value - Bedding (Tilt) corr.
 codefol$ = "B"
 codelin$ = "F"

 pi = 3.14159265358979#
 deg = 180 / pi
 Nrep = 0                     ' current N. of repetition
 MaxRep = 10                  ' max. number of repetition
 Repf = 0                     ' repeat flag (for curr. pos.)
 RepModef = 0                 ' repeat mode (for curr. spec.)
 dat = 0: E% = 0
 NR = 880: mtime = 0: mtime$ = "-??"
 mmode = 1
 holpos = 1
 ghdef = 6.8: fihdef = -38
 gldef = 72: fildef = 72

 CLS
 KEY OFF
 PRINT
 PRINT "***  REMA6 "; version$; " ***"
 PRINT
 PRINT "The program  REMA6 processes JR-6 / JR-6A spinner magnetometer measurement"
 PRINT "of remanent magnetization of rock specimen using Manual standard holder,"
 PRINT "Semi-automatic holder or Automatic holder in case of JR-6A magnetometer."
 PRINT
 PRINT "Program generates the Text file with ext. [.TXT] in the same format as"
 PRINT "displayed on the screen and the Data file [.JR6], which can be used as "
 PRINT "an input file for the Advanced Data Interpretation by software REMASOFT. "
 PRINT
 PRINT "The Geological file [.GED] can be optionally used to input field data."
 PRINT "It is assumed the Geological file is located in the same directory as"
 PRINT "the Data file, but it is not necessary."
 PRINT
 PRINT "Program uses Configuration file PARJR6.CFG which should be present in the"
 PRINT "current directory. This file contains calibration constants and configuration"
 PRINT "parameters. In case the file is missing, it can be created automatically."

	 LOCATE 22, 1
	 PRINT "--> Switch ON the Spinner Magnetometer"
	 PRINT "--> Press any key to continue "
	 q$ = INPUT$(1)

'   Declaration of all combination <ESC> :
	 KEY 16, CHR$(0) + CHR$(1)     ' Esc
	 KEY 17, CHR$(32) + CHR$(1)    ' + NumLock ON
	 KEY 18, CHR$(64) + CHR$(1)    ' + CapsLock ON
	 KEY 19, CHR$(96) + CHR$(1)    ' + NumLock + CapsLock ON

'****************

start:
 CLS
 GOSUB keysblankoff
 GOSUB readconfig

Selectpath:
	 CLS
	 PRINT "*** Selection of the destination of the data file "
	 PRINT
	 PRINT "*** The current path is  "; CURDIR$
	 PRINT "*** Press <CR> to confirm or Enter a new path  "
	 PRINT
	 PRINT "--> Enter Path of the Data file (drive:\dir1\dir2\...) ";
	 INPUT path$
	 IF path$ = "!" THEN GOTO skok
	 path$ = UCASE$(path$)
	 IF path$ = "" THEN path$ = CURDIR$
	 IF LEN(path$) = 1 THEN path$ = path$ + ":"
	 IF RIGHT$(path$, 1) <> "\" THEN path$ = path$ + "\"
	 drive$ = LEFT$(path$, 1)
	 IF drive$ = "A" OR drive$ = "B" THEN
		PRINT "--> Insert diskette for drive "; drive$; " and press any key"
		q$ = INPUT$(1)
		LOCATE CSRLIN - 1, 1
		PRINT SPACE$(79);
		GOSUB testprot
	 END IF
	 PRINT
	 ilinz = CSRLIN
	 IF LEN(DIR$(path$ + "*.JR6")) <> 0 THEN
		 FILES path$ + "*.JR6"
	 ELSE
		 ON ERROR GOTO invalidpath
		 FILES path$
		 ON ERROR GOTO 0
	 END IF
	 ilink = CSRLIN
	 LOCATE ilinz, 1
	 PRINT SPACE$(79);
	 LOCATE ilinz, 1
	 PRINT "Directory  "; path$
	 LOCATE ilink - 1, 1
	 PRINT SPACE$(79);
	 PRINT

sname:
	 PRINT "*** To change path enter #"
	 INPUT "--> Enter name of data output file (max.8 chars) "; nam$
	 IF nam$ = "#" THEN GOTO Selectpath
	 IF LEN(nam$) < 1 OR LEN(nam$) > 8 THEN GOTO sname

	 Filedata$ = path$ + nam$ + ".JR6"
	 Filetxt$ = path$ + nam$ + ".TXT"
	 OPEN "R", #2, Filedata$, 82
	 FOR Rec% = 1 TO 1000
	 FIELD #2, 10 AS F1$, 8 AS F2$, 6 AS F3$, 6 AS F4$, 6 AS F5$, 4 AS F6$, 4 AS F7$, 4 AS F8$, 4 AS F9$, 4 AS F10$, 4 AS F11$, 4 AS F12$, 3 AS F13$, 3 AS F14$, 3 AS F15$, 3 AS F16$, 4 AS F17$, 2 AS F18$
	 GET #2, Rec%
	 IF F1$ = STRING$(10, 0) THEN GOTO s2
	 NEXT Rec%
	 PRINT "### Invalid Data File : "; Filedata$
	 CLOSE
	 END

s2:
	 Rec% = Rec% - 1
	 IF Rec% = 0 THEN GOTO cont
	 PRINT
	 PRINT "*** File already exist"
	 PRINT "--> Append to File :  "; Filedata$; "  [Y/N] ? ";
s3:
	 q$ = INPUT$(1): q$ = UCASE$(q$)
	 IF q$ = "Y" THEN
		 PRINT q$
		 appendf = -1
		 GOTO cont
	 ELSEIF q$ <> "N" THEN
		BEEP
		GOTO s3
	 ELSE
		PRINT q$
		CLOSE #2
		PRINT
		GOTO sname
	 END IF

'---------------------------------------------

cont:
	 IF appendf THEN
		 dum$ = ""
		 ON ERROR GOTO missing
		 OPEN Filetxt$ FOR INPUT AS #4
		 IF NOT EOF(4) THEN LINE INPUT #4, dum$
		 CLOSE #4
nofile:
		 ON ERROR GOTO 0
		 IF dum$ = "" THEN
			 PRINT
			 PRINT "Warning "
			 PRINT "*** Associate Text file  "; Filetxt$; "  is empty but Append was set"
			 PRINT "*** Text file was probably removed"
			 PRINT "*** You can link the text files later or copy the removed Text file now"
			 PRINT
			 BEEP
			 PRINT "--> Continue or Exit program [C/E] ? "
zpet10:
			 q$ = INPUT$(1)
			 IF UCASE$(q$) = "E" THEN
				 GOTO mk10a
			 ELSEIF UCASE$(q$) <> "C" THEN
				 BEEP
				 GOTO zpet10
			 END IF
		 END IF
	 END IF
	 OPEN Filetxt$ FOR APPEND AS #3
	 geoldata = 0
	 GOSUB selway
	 CLS
	 PRINT
	 PRINT "*** Data File           "; Filedata$
	 PRINT
	 PRINT "*** Text File           "; Filetxt$
	 PRINT
	 IF geoldata THEN
		 PRINT "*** Geological File     "; Filegeo$
	 END IF
	 LOCATE 23, 1
	 PRINT SPACE$(79)
	 LOCATE 23, 1
	 PRINT "--> Press any key to continue "
	 q$ = INPUT$(1)

'-------------------------------------------------------------

skok:
'GOSUB enesc
GOSUB mainpage
GOSUB serport

mainloop:
 
 retf = 0: RepModef = 0: Repf = 0: Nrep = 0
 GOSUB testorparam

 IF badopf THEN GOSUB mk3
 IF Vact < .01 OR Vact > 15.5 THEN GOSUB mk4
 IF ETALON < 5 OR ETALON > 20 THEN GOSUB mk6

 KEY 1, " MEAS ": ON KEY(1) GOSUB mk1
 IF commf AND NOT nonor THEN
	 KEY 2, " Tecto": ON KEY(2) GOSUB mk2commok
 ELSE
	 KEY 2, " Com ": ON KEY(2) GOSUB mk2
 END IF
 KEY 3, " OPar ": ON KEY(3) GOSUB mk3
 KEY 4, " SVol ": ON KEY(4) GOSUB mk4
 KEY 5, "  CAL ": ON KEY(5) GOSUB mk5
 KEY 6, "  Std ": ON KEY(6) GOSUB mk6
 KEY 7, " HType": ON KEY(7) GOSUB mk7
 KEY 8, " Speed": ON KEY(8) GOSUB mk8
 KEY 9, " CORR ": ON KEY(9) GOSUB mk9
 KEY 10, " Exit ": ON KEY(10) GOSUB mk10

 GOSUB keyson

 IF commf THEN
	KEY 2, " Tecto": ON KEY(2) GOSUB mk2commok
	GOSUB mainpage
	LOCATE 20, 1
	IF Htype = 0 AND NOT simul THEN
		KEY 1, "     ": KEY(1) OFF
		KEY 5, "     ": KEY(5) OFF
		KEY 9, "     ": KEY(9) OFF
		mmode = 0
		automf = 0
		Clearbuff
		cmd$ = "r"
		PRINT #1, cmd$;
		LOCATE 2, 52
		COLOR 15
		PRINT "       *** Setting Rest Pos."
		Pause .5
		COLOR 7
		GOSUB receive
		mes1$ = a$
		LOCATE 1, 52
		PRINT SPACE$(28);
		Pause .5
		Clearbuff
		cmd$ = "y"              ' mainloop
		GOSUB diesc
		PRINT #1, cmd$;
		LOCATE 2, 52
		COLOR 15
		PRINT "       *** Setting Init Pos."
		Pause .5
		GOSUB receive
		mes2$ = a$
		IF mes1$ <> "Ok" THEN
			LOCATE 1, 52
			PRINT SPACE$(28);
			LOCATE 1, 80 - LEN(mes1$)
			COLOR 10
			PRINT mes1$;                         ' disp instrument message
			COLOR 7
		ELSE
			LOCATE 1, 52
			PRINT SPACE$(28);
			IF LEN(mes2$) > 28 THEN mes2$ = RIGHT$(mes2$, 28)
			LOCATE 1, 80 - LEN(mes2$)
			COLOR 10
			PRINT mes2$;                         ' disp instrument message
			COLOR 7
		END IF
		LOCATE 20, 1
		COLOR 7
		IF mes1$ = "Ok" AND mes2$ = "Ok" THEN
			PRINT " *** JR-6A READY"
			automf = -1
			IF holpos <> 1 THEN
				LOCATE 1, 52
				BEEP
				COLOR 15
				PRINT "Previous routine interrupted "
				LOCATE 2, 52
				COLOR 12
				PRINT "   --> Reset holder position "
				COLOR 7
				Pause 10
			END IF
			holpos = 1
		ELSE
			Htype = 1
			BEEP
			PRINT " ### Automatic Manipulator Failed"
			PRINT " *** Semi-Automatic Holder is set"
			LOCATE 9, 1
			PRINT " SemiAuto";
			mmode = 1               ' LMP=4 for manual holder, if change later
		END IF
	ELSE
		PRINT " *** JR-6 READY"
	END IF

	IF (Htype < 2 AND automf) OR Htype = 2 THEN
		IF Speed = 0 THEN cmd$ = "h" ELSE cmd$ = "l"
		PRINT #1, cmd$;
		GOSUB receive
		LOCATE 1, 52
		PRINT SPACE$(28);
		IF LEN(a$) > 28 THEN a$ = RIGHT$(a$, 28)
		LOCATE 1, 80 - LEN(a$)
		COLOR 10
		PRINT a$;
		COLOR 7
		IF simul THEN
			P = 9
			cmd$ = "a"
			PRINT #1, cmd$;
			GOSUB receive
			LOCATE 1, 52
			PRINT SPACE$(28);
			IF LEN(a$) > 28 THEN a$ = RIGHT$(a$, 28)
			LOCATE 1, 80 - LEN(a$)
			COLOR 10
			PRINT a$;
			COLOR 7
		END IF
	END IF

	KEY 1, " MEAS ": KEY(1) ON
	KEY 5, "  CAL ": KEY(5) ON
	KEY 9, " CORR ": KEY(9) ON
	LOCATE 23, 1
	PRINT " --> Check the set parameters"
 ELSE
	GOSUB keysblankoff
	KEY 2, " Com ": KEY(2) ON: ON KEY(2) GOSUB mk2
	KEY 10, " Exit ": KEY(10) ON: ON KEY(10) GOSUB mk10
	LOCATE 23, 1
	PRINT " --> Press F2 Key to check again"
 END IF

 IF gnom = 1! THEN
	 LOCATE 23, 1
	 PRINT SPACE$(79)
	 LOCATE 23, 1
	 PRINT " --> Execute Calibration"
	 BEEP
	 KEY 1, "      ": KEY(1) OFF
	 KEY 9, "      ": KEY(9) OFF
 END IF

 IF nonor OR geoldata THEN
	 IF commf THEN
		 KEY 2, "      "
		 KEY(2) OFF
	 END IF
	 KEY 3, "      "
	 KEY(3) OFF
 END IF

 IF Htype < 2 AND Stype = 1 AND commf THEN
	 LOCATE 21, 1
	 PRINT " *** Position of the CUBIC Specimen in the Holder - Arrow Left and Up"
 END IF
 
waitloop:
	 IF Htype = 0 AND escaf AND specrout THEN
		 KEY 1, " Start": KEY(1) ON
		 escaf = 0
	 END IF
	 escf = 0
	 KEY 10, " Exit": KEY(10) ON        ' ver. 1.3
	 IF retf = 1 THEN GOTO mainloop ELSE GOTO waitloop


mainpage:
specrout = 0
VIEW PRINT 1 TO 24
CLS
LOCATE 1, 1
COLOR 7
PRINT " --> Select Function Key"
LOCATE 3, 1
PRINT " Current setting"; TAB(30); doc(1)
IF commf AND NOT nonor THEN
	SELECT CASE Tectcorr
		CASE 0
			PRINT " None";
		CASE 1
			IF geoldata THEN PRINT "     ";  ELSE PRINT " Tilt";
		CASE 2
			PRINT " Full";
	END SELECT
	PRINT TAB(30); doc2
ELSE
	PRINT USING "  # &"; sport; TAB(30); doc(2)
END IF
PRINT USING " ##  ##  ##  ## &"; z(1); z(2); z(3); z(4); TAB(30); doc(3)
PRINT USING " ##.## &"; Vact; TAB(30); doc(4)
PRINT TAB(30); doc(5)
PRINT USING " ##.##  ##.## &"; ETALON; Vetal; TAB(30); doc(6)
SELECT CASE Htype
  CASE 0
	  PRINT " Auto";
	  allpos = 3
  CASE 1
	  PRINT " SemiAuto";
	  allpos = 3
  CASE 2
	  PRINT " Manual";
	  allpos = 4
  CASE ELSE
	  Htype = 2
END SELECT
PRINT TAB(30); doc1(7)
IF Htype < 2 THEN
	 SELECT CASE Stype
		 CASE 0
		PRINT " Cylinder";
		 CASE 1
		PRINT " Cube";
		 CASE ELSE
	  PRINT " Cylinder";
	  Stype = 0
	 END SELECT
	 PRINT TAB(30); doc7p
END IF
SELECT CASE Speed
  CASE 0
	  PRINT " High";
	  Speed$ = " H"
  CASE 1
	  PRINT " Low";
	  Speed$ = " L"
  CASE ELSE
	  PRINT " High";
	  Speed = 0
	  Speed$ = " H"
END SELECT
PRINT TAB(30); doc1(8)
PRINT TAB(30); doc(9)
PRINT TAB(30); doc(10)
LOCATE 15, 1
PRINT " Calibration constants";
PRINT TAB(45); "Holder remanent values"
PRINT USING " Gain ###.### Phase +###.##"; gnom; fi;
IF Htype = 2 THEN
	xt = Hm1
	yt = Hm2
	rg = 0
	GOSUB zobraz
	Hmrg = rg - 5
	PRINT TAB(45); "H0";
	PRINT USING "  +##.## +##.##  E+#"; xu; yu; Hmrg
ELSE
	FOR cp = 1 TO 3
		 xt = Ha(2 * cp - 1)
		 yt = Ha(2 * cp)
		 rg = 0
		 GOSUB zobraz
		 Harg(cp) = rg - 5
		 PRINT TAB(45); "H" + RIGHT$(STR$(cp), 1);
		 PRINT USING "  +##.## +##.##  E+#"; xu; yu; Harg(cp)
	NEXT cp
END IF
RETURN


'******************************************* m keys ************************


'***** measurement routine
mk1:

 GOSUB specimen

 ON KEY(1) GOSUB ak1
 ON KEY(2) GOSUB ak2
 ON KEY(3) GOSUB ak3
 ON KEY(4) GOSUB ak4
 ON KEY(5) GOSUB ak5
 ON KEY(6) GOSUB ak6
 ON KEY(7) GOSUB ak7

 KEY 2, "      ": KEY(2) OFF
 KEY 3, "      ": KEY(3) OFF
 KEY 4, "      ": KEY(4) OFF
 KEY 5, "      ": KEY(5) OFF
 KEY 6, "      ": KEY(6) OFF
 KEY 7, "      ": KEY(7) OFF

 IF Htype = 0 THEN
	 KEY 1, " Start"
 ELSEIF Htype = 1 THEN
	 KEY 1, "  P1  "
	 KEY 2, "  P2  ": KEY(2) ON
	 KEY 3, "  P3  ": KEY(3) ON
 ELSE
	 KEY 1, "  P1  "
	 KEY 2, "  P2  ": KEY(2) ON
	 KEY 7, "  LMP ": KEY(7) ON
	 IF mmode > 0 THEN
		 KEY 3, "  P3  ": KEY(3) ON
		 KEY 4, "  P4  ": KEY(4) ON
	 END IF
	 IF mmode > 1 THEN
		 KEY 5, "  P5  ": KEY(5) ON
		 KEY 6, "  P6  ": KEY(6) ON
	END IF
 END IF
 IF RepModef = 0 THEN
	 KEY 8, "  Rep "
	 ON KEY(8) GOSUB ak8
 END IF

 KEY 9, " MTime": ON KEY(9) GOSUB ak9
 KEY 10, " Kill ": ON KEY(10) GOSUB ak10

 IF mtime$ = "   " THEN
	 cmd$ = CHR$(26)
 ELSEIF mtime$ = "-LT" THEN
	 cmd$ = CHR$(24)
 ELSEIF mtime$ = "-ST" THEN
	 cmd$ = CHR$(25)
 END IF
 PRINT #1, cmd$;
 GOSUB receive
RETURN


'***** select Tecto correction - active if serial comm. is Ok only
mk2commok:

Tectcorr = Tectcorr + 1
IF Tectcorr = 3 THEN Tectcorr = 0
GOSUB mainpage
retf = 1
RETURN


'***** serial comunication port number
mk2:

CLOSE #1
retf = 1
VIEW PRINT 20 TO 24
CLS
LOCATE 20, 1
IF sport = 1 THEN sport = 2 ELSE sport = 1
PRINT " --> Change to COM" + RIGHT$(STR$(sport), 1); " [Y/N] ?"
zpet:
q$ = INPUT$(1)
IF UCASE$(q$) = "Y" THEN
	GOSUB writeconfig
	GOSUB mainpage
	GOSUB serport
	RETURN
ELSEIF UCASE$(q$) = "N" THEN
	IF sport = 1 THEN sport = 2 ELSE sport = 1
	GOSUB mainpage
	GOSUB serport
	RETURN
ELSEIF UCASE$(q$) <> "N" THEN
  BEEP
  GOTO zpet
END IF

serport:
sport$ = RIGHT$(STR$(sport), 1)
ON ERROR GOTO notexist
OPEN "COM" + sport$ + ":19200,n,8,1,cs,ds" FOR RANDOM AS #1
ON ERROR GOTO 0
Clearbuff
cmd$ = "a"
PRINT #1, cmd$;
star = TIMER
w$ = ""
commf = 0
IF NOT simul THEN
 DO
	IF LOC(1) > 0 THEN
		w1$ = INPUT$(1, #1)
		w$ = w$ + w1$
		IF w1$ = CHR$(13) THEN EXIT DO
	END IF
	wt = TIMER - star
 LOOP WHILE wt < 3
ELSE                                               ' !!!!!
 wt = 1
 a$ = "Auto range" + CHR$(13)
END IF
IF wt >= 3 THEN
	CLOSE #1
	LOCATE 20, 1
	PRINT " ### RS-232 Communication Error"
	GOSUB beep3x
	PRINT " *** Current communication port : COM" + sport$
	PRINT " --> Check the serial cable connection and switch ON the JR-6"
ELSE
	commf = -1


	cmd$ = CHR$(16)      ' firmware program version
	PRINT #1, cmd$;
	GOSUB receive
	PRINT #1, CHR$(20);
	PRINT #1, CHR$(3);
	Pause .8
	Clearbuff

	cmd$ = CHR$(26)      ' normal time
	PRINT #1, cmd$;
	GOSUB receive
	IF LEFT$(a$, 6) = "Normal" THEN mtime$ = "   "
	NR = 880
	Pause .5

END IF
RETURN


'***** orientation parameters
mk3:
VIEW PRINT 20 TO 24
CLS
VIEW PRINT 1 TO 24
GOSUB keysoff
IF badopf THEN BEEP
FOR op = 1 TO 4
	IF badopf THEN
		LOCATE 19, 1
		PRINT " ### Illegal Orientation Parameter"
		goodf = -1
		GOSUB testop
		IF goodf THEN GOTO nextpar
	END IF
zpet5:
  LOCATE 19 + op, 1
  PRINT SPACE$(79);
  LOCATE 19 + op, 1
  PRINT " Enter orientation parameter P" + RIGHT$(STR$(op), 1) + " ";
  INPUT z(op)
  goodf = -1
  GOSUB testop
  IF NOT goodf THEN
	  BEEP
	  GOTO zpet5
  END IF
nextpar:
NEXT op
GOSUB testorparam
GOSUB writeconfig
GOSUB mainpage
retf = 1
RETURN

testop:
IF op = 1 OR op = 3 THEN
	IF z(op) < 3 OR z(op) > 12 OR z(op) MOD 3 <> 0 THEN goodf = 0
ELSE
	IF z(op) <> 0 AND z(op) <> 90 THEN goodf = 0
END IF
RETURN

testorparam:
goodf = -1
badopf = -1
FOR op = 1 TO 4
	GOSUB testop
NEXT op
IF goodf THEN
	IP1 = z(1): IP2 = z(2): IP3 = z(3): IP4 = z(4)
	badopf = 0
END IF
RETURN

printinv:
PRINT " Illegal value" + SPACE$(60);
BEEP
Pause 1
RETURN

'***** specimen volume
mk4:

VIEW PRINT 20 TO 24
CLS
VIEW PRINT 1 TO 24
GOSUB keysoff
LOCATE 20, 1
Vold = Vact
PRINT " Enter specimen volume [ccm] ";
INPUT Vact
IF Vact < .01 OR Vact > 15.5 THEN
	LOCATE 20, 1
	GOSUB printinv
	GOTO mk4
END IF
Hm1 = Hm1 * Vold / Vact
Hm2 = Hm2 * Vold / Vact
FOR cp = 1 TO 6
  Ha(cp) = Ha(cp) * Vold / Vact
NEXT cp
GOSUB writeconfig
GOSUB mainpage
retf = 1
RETURN


'***** calibration
mk5:

cmd$ = CHR$(26)
PRINT #1, cmd$;      ' normal time
GOSUB receive
IF LEFT$(a$, 6) = "Normal" THEN mtime$ = "   "
NR = 880

FOR i = 1 TO 12      ' blank data
	 T(i) = 9999
NEXT i

	 VIEW PRINT 19 TO 24
	 CLS
	 VIEW PRINT 1 TO 24
	 etatot = ETALON * Vetal
	 P = 1
	 IF simul THEN P = 7            ' !!!!!
	 Clearbuff
	 cmd$ = "g"
	 PRINT #1, cmd$;
				COLOR 14
				LOCATE 2, 52
				PRINT "              <Esc>  to stop ";
				COLOR 7
	 LOCATE 1, 52
	 COLOR 15
	 PRINT "  *** CALIBRATION IS RUNNING "
	 COLOR 7
	 GOSUB receive
	 IF escf OR nodataf THEN RETURN
	 LOCATE 2, 52
	 PRINT SPACE$(28);
	 cmd$ = "i"
	 PRINT #1, cmd$;
		LOCATE 2, 52
		COLOR 15
		PRINT "       *** Setting Main Pos."
		COLOR 7
	 IF NOT simul THEN GOSUB receive
	 IF escf THEN RETURN

	 Vsav = Vact
	 Vact = Vetal
	 yy = x
	 xx = y
	 GOSUB arctg2
	 fi = atn2
	 amp = SQR(x * x + y * y)
	 gnom = etatot / (amp + 1E-12)          ' instrument gain for spec. of 1 ccm ver.1.4
	 IF gnom > 1000 THEN gnom = 999.999
	 GOSUB keysoff
	 LOCATE 19, 1
	 PRINT USING " Gain ###.### Phase +###.##"; gnom; fi
	 GOSUB rotgain
	 GOSUB zobraz
	 LOCATE 21, 1
	 PRINT USING " CAL: +##.## +##.##  E+#"; xu; yu; rg - 5
	 Vact = Vsav
	 calerrf = 0
	 SELECT CASE Speed
	 CASE 0
		IF gnom < .85 * ghdef OR gnom > 1.15 * ghdef OR ABS(fi - fihdef) > 25 THEN
			calerrf = -1
			GOSUB readconfig
			Gnomh = 1!
		END IF
	 CASE 1
		IF gnom < .85 * gldef OR gnom > 1.15 * gldef OR ABS(fi - fildef) > 25 THEN
			calerrf = -1
			GOSUB readconfig
			Gnoml = 1!
		END IF
	 END SELECT
	 IF calerrf THEN
		 LOCATE 21, 1
		 PRINT " ### Calibration error        "
		 BEEP
		 LOCATE 23, 1
		 PRINT " --> Press any key to continue"
		 q$ = INPUT$(1)
		 gnom = 1!
		 GOSUB writeconfig
		 GOSUB mainpage
		 retf = 1
		 RETURN
	 END IF
zpet1:
	 LOCATE 23, 1
	 PRINT " Save new values [Y/N] ?"
	 q$ = INPUT$(1)
	 IF UCASE$(q$) = "Y" THEN
		 IF Speed = 0 THEN
			 Fih = fi
			 Gnomh = gnom
		 ELSEIF Speed = 1 THEN
			 Fil = fi
			 Gnoml = gnom
		 END IF
		 GOSUB writeconfig
	 ELSEIF UCASE$(q$) <> "N" THEN
		 BEEP
		 GOTO zpet1
	 END IF
	 GOSUB readconfig
	 GOSUB mainpage
	 retf = 1
RETURN


'****** calibration standard magnetization and volume
mk6:
VIEW PRINT 20 TO 24
CLS
VIEW PRINT 1 TO 24
GOSUB keysoff
zpet6:
  LOCATE 21, 1
  PRINT SPACE$(70);
  LOCATE 21, 1
  PRINT " Enter Calibration Standard Magnetization [A/m] ";
  INPUT ETALON
  IF ETALON < 5 OR ETALON > 20 THEN
	  LOCATE 21, 1
	  GOSUB printinv
	  GOTO zpet6
  END IF
zpet7:
  LOCATE 22, 1
  PRINT SPACE$(70);
  LOCATE 22, 1
  PRINT " Enter Calibration Standard Volume [ccm]        ";
  INPUT Vetal
  IF Vetal < 1 OR Vetal > 16 THEN
	  LOCATE 22, 1
	  GOSUB printinv
	  GOTO zpet7
  END IF
gnom = 1!
Gnomh = gnom
Gnoml = gnom
GOSUB writeconfig
GOSUB mainpage
retf = 1
RETURN


'***** holder type
mk7:
VIEW PRINT 20 TO 24
CLS
Htype = Htype + 1
IF Htype = 3 THEN Htype = 0
mmode = 1
GOSUB mainpage
VIEW PRINT 20 TO 24
CLS
IF Htype < 2 THEN
 mmode = 0
 GOSUB keysblankoff
 KEY 7, "  Cyl "
 KEY 8, "  Cub "
 ON KEY(7) GOSUB mk7aux
 KEY(7) ON
 ON KEY(8) GOSUB mk8aux
 KEY(8) ON
 LOCATE 21, 1
 IF Htype = 0 THEN
	 PRINT " *** Auto Holder is selected"
 ELSE
	 PRINT " *** SemiAuto Holder is selected"
	 automf = -1
 END IF
 PRINT " --> Select Specimen Type"
 keypress = 0
 DO
	IF keypress THEN EXIT DO
 LOOP WHILE -1
END IF
retf = 1
VIEW PRINT 20 TO 24
CLS
GOSUB mainpage
RETURN

mk7aux:
Stype = 0
keypress = -1
RETURN

mk8aux:
VIEW PRINT 20 TO 24
CLS
GOSUB keysblankoff
zpet2:
LOCATE 21, 1
PRINT " --> Do you really measure CUBIC specimen [Y/N] ?"
q$ = INPUT$(1)
IF UCASE$(q$) = "Y" THEN
	Stype = 1
ELSEIF UCASE$(q$) <> "N" THEN
	BEEP
	GOTO zpet2
END IF
keypress = -1
RETURN

'***** speed
mk8:
VIEW PRINT 20 TO 24
CLS
VIEW PRINT 1 TO 24
GOSUB keysoff
zpet3:
LOCATE 20, 1
PRINT " *** Warning : Changing the speed will zero holder values"
PRINT " --> Change the speed [Y/N] ?"
q$ = INPUT$(1)
IF UCASE$(q$) = "Y" THEN
	IF Speed = 1 THEN Speed = 0 ELSE Speed = 1
	LOCATE 21, 1
	PRINT SPACE$(79)
	SELECT CASE Speed
	  CASE 0
		 cmd$ = "h"
		 PRINT #1, cmd$;
		 GOSUB receive
		 fi = Fih
		 gnom = Gnomh
		 PRINT " *** High Speed is selected";
		 Speed$ = " H"
	  CASE 1
		 cmd$ = "l"
		 PRINT #1, cmd$;
		 GOSUB receive
		 fi = Fil
		 gnom = Gnoml
		 PRINT " *** Low Speed is selected";
		 Speed$ = " L"
	END SELECT
	Hm1 = 0
	Hm2 = 0
	FOR cp = 1 TO 6
		Ha(cp) = 0
	NEXT cp
ELSEIF UCASE$(q$) <> "N" THEN
	BEEP
	GOTO zpet3
END IF
Pause .5
GOSUB writeconfig
GOSUB mainpage
retf = 1
RETURN


'***** correction for holder

mk9:

KEY 10, "      ": KEY(10) OFF                   ' ver 1.3
cmd$ = CHR$(26)
PRINT #1, cmd$;      ' normal time
GOSUB receive
IF LEFT$(a$, 6) = "Normal" THEN mtime$ = "   "
NR = 880

  FOR i = 1 TO 12                               ' blank data
	 T(i) = 9999
  NEXT i

  VIEW PRINT 20 TO 24
  CLS
  VIEW PRINT 1 TO 24
  IF Htype = 0 AND NOT simul THEN
	  Clearbuff
	  cmd$ = "r"
	  PRINT #1, cmd$;
		LOCATE 2, 52
		COLOR 15
		PRINT "       *** Setting Rest Pos."
	  GOSUB receive
	  IF escf OR errf THEN GOTO escerr
	  cmd$ = "y"                                 ' correction
	  GOSUB diesc
	  PRINT #1, cmd$;
		LOCATE 2, 52
		COLOR 15
		PRINT "       *** Setting Init Pos."
	  GOSUB receive
	  IF escf OR errf THEN GOTO escerr
			IF holpos <> 1 THEN
				LOCATE 1, 52
				BEEP
				COLOR 12
				PRINT "   --> Reset holder position"
				LOCATE 2, 52
				COLOR 14
				PRINT "            <CR> to continue"
				COLOR 7
				DO
					q$ = INPUT$(1)
				LOOP WHILE q$ <> CHR$(13)
			END IF
	  holpos = 1
  END IF
  IF Htype = 2 THEN allcorr = 1 ELSE allcorr = 3

  corrf = 0
  FOR cp = 1 TO allcorr
	 P = cp
	 Clearbuff
	 cmd$ = "g"
	 IF simul THEN cmd$ = RIGHT$(STR$(P), 1)        ' !!!!!
	 PRINT #1, cmd$;
				COLOR 14
				LOCATE 2, 52
				PRINT "              <Esc>  to stop ";
				COLOR 7
	 LOCATE 1, 52
	 COLOR 15
	 PRINT "   *** CORRECTION IS RUNNING "
	 COLOR 7
	 GOSUB receive
	 IF escf OR nodataf THEN GOTO escerr
	 GOSUB rotgain
	 IF amp > .0002 THEN corrf = -1
	 GOSUB zobraz
	 LOCATE 20, 1
	 PRINT " Holder correction";
	 LOCATE 19 + cp, 45
	 IF Htype = 2 THEN
		 Hm1x = xt
		 Hm2x = yt
		 Hmrg = rg - 5
		 PRINT "H0";
		 PRINT USING "  +##.## +##.##  E+#"; xu; yu; Hmrg

	 ELSE
		 Hax(2 * cp - 1) = xt
		 Hax(2 * cp) = yt
		 Harg(cp) = rg - 5
		 PRINT "H" + RIGHT$(STR$(cp), 1);
		 PRINT USING "  +##.## +##.##  E+#"; xu; yu; Harg(cp)
		 'PRINT USING "    +#.##^^^^     +#.##^^^^ &"; Hax(2 * cp - 1); Hax(2 * cp)
		 IF Htype = 1 AND cp < 3 THEN
			 LOCATE 2, 52
			 PRINT SPACE$(28);
			 cmd$ = "i"
			 PRINT #1, cmd$;
			 IF NOT simul THEN GOSUB receive
			 IF escf THEN RETURN
			 LOCATE 23, 1
			 PRINT " --> Set next Holder Position, press <CR> to continue"
zpet15:   q$ = INPUT$(1)
			 IF q$ <> CHR$(13) THEN GOTO zpet15
			 LOCATE 23, 1
			 PRINT SPACE$(80);
		 END IF
	 END IF
	 IF Htype = 0 AND NOT simul THEN
		 LOCATE 2, 52
		 PRINT SPACE$(28);
		 cmd$ = "i"
		 PRINT #1, cmd$;
			LOCATE 2, 52
			COLOR 15
			PRINT "       *** Setting Main Pos."
			COLOR 7
		 GOSUB receive
		 IF escf OR errf THEN GOTO escerr
			Pause .5
		 cmd$ = "w"
		 PRINT #1, cmd$;
			LOCATE 2, 52
			COLOR 15
			PRINT "       *** Setting Work Pos."
			COLOR 7
		 GOSUB receive
		 IF escf OR errf THEN GOTO escerr
			Pause .5
		 cmd$ = "t"
		 GOSUB diesc
		 PRINT #1, cmd$;
			LOCATE 2, 52
			COLOR 15
			PRINT "       *** Setting Next Pos."
			COLOR 7
		 GOSUB receive
		 IF escf OR errf THEN GOTO escerr
			Pause .5
		 holpos = holpos + 1
		 IF holpos = 4 THEN holpos = 1
		 cmd$ = "r"
		 PRINT #1, cmd$;
			LOCATE 2, 52
			COLOR 15
			PRINT "       *** Setting Rest Pos."
			COLOR 7
		 GOSUB receive
		 IF escf OR errf THEN GOTO escerr
	 END IF
  NEXT cp
  GOSUB keysoff
  LOCATE 2, 52
  PRINT SPACE$(28);
  cmd$ = "i"
  PRINT #1, cmd$;
  IF NOT simul THEN GOSUB receive

zpet4:
		 LOCATE 23, 1
		 IF corrf THEN
			 PRINT " Warning : New values are too high, save them anyway [Y/N] ?"
		 ELSE
			 PRINT " Save new values [Y/N] ?"
		 END IF
		 q$ = INPUT$(1)
		 IF UCASE$(q$) = "Y" THEN
			 IF Htype = 2 THEN
				 Hm1 = Hm1x
				 Hm2 = Hm2x
			 ELSE
				 FOR i = 1 TO 6
					 Ha(i) = Hax(i)
				 NEXT i
			 END IF
			 GOSUB writeconfig
		 ELSEIF UCASE$(q$) <> "N" THEN
			 BEEP
			 GOTO zpet4
		 END IF
		 GOSUB mainpage
		 retf = 1
RETURN


'***** exit program
mk10:
LOCATE 1, 52
PRINT SPACE$(28);
LOCATE 23, 52
PRINT "    --> Confirm Exit [Y/N] ? "
BEEP
q$ = INPUT$(1)                        '******
'q$ = "Y"                               '******
IF UCASE$(q$) = "Y" THEN
	VIEW PRINT 20 TO 24
	CLS
	  GOSUB keysoff
	  cmd$ = CHR$(27)
	  IF commf THEN PRINT #1, cmd$;             'send STOP in case motor is on
mk10a:
	  GOSUB writeconfig
	  SOUND 3000, 10
	  COLOR 15
	  LOCATE 23, 52
	  PRINT "      *** Program terminated";
	  COLOR 7
	  VIEW PRINT 1 TO 25
	  COLOR 7, 0
	  CLOSE
	  GOSUB keysoff
	  END

ELSE
	LOCATE 23, 52
	PRINT SPACE$(28);
END IF
RETURN



'***** stop measurement, clear repeat  <ESC> key
Esc:
	 GOSUB diesc
	 LOCATE 2, 52
	 PRINT SPACE$(28);
	 clin = CSRLIN: ccol = POS(0)
	 IF Repf = 1 THEN
		 PRINT #1, "x";
		 Repf = 0
		 LOCATE 2, 52
		 COLOR 14
		 PRINT "              <Esc>  to stop ";
		 COLOR 15
		 LOCATE 23, 52
		 PRINT "   *** Repetition terminated"
		 BEEP
		 LOCATE clin, ccol
		 COLOR 7
		 GOSUB enesc
		 RETURN
	 END IF
	 cmd$ = CHR$(27)
	 PRINT #1, cmd$;
	 escf = -1
	 FOR i = 1 TO 10
		 KEY(i) OFF
	 NEXT i
	 COLOR 15
	 LOCATE 23, 52
	 PRINT "    *** Execution terminated"
	 BEEP
	 LOCATE clin, ccol
	 COLOR 7
	 Pause .5
	 IF simul THEN P = 0                                '!!!!!
	 GOSUB receive
	 'GOSUB enesc
	 Repf = 0
	 escf = -1
	 escaf = -1
 KEY(1) ON
 KEY(10) ON
 IF Rm = 9999 THEN
	 IF Htype = 2 THEN KEY(7) ON
	 KEY(9) ON
 END IF
 IF Htype = 1 THEN        ' semi-auto
	 KEY(2) ON
	 KEY(3) ON
 END IF
 IF Htype = 2 THEN         ' manual
	 KEY(2) ON
	 IF mmode > 0 THEN
		 KEY(3) ON
		 KEY(4) ON
	 END IF
	 IF mmode > 1 THEN
		 KEY(5) ON
		 KEY(6) ON
	 END IF
 END IF
 IF NOT specrout THEN
	 FOR i = 1 TO 10
		 KEY(i) ON
	 NEXT i
 END IF
RETURN

'********************************************* a keys *********************

ak1: P = 1: GOTO mp1
ak2: P = 2: GOTO mpj
ak3: P = 3: GOTO mpj
ak4: P = 4: GOTO mpj
ak5: P = 5: GOTO mpj
ak6: P = 6: GOTO mpj

mp1:
KEY 10, "      ": KEY(10) OFF         ' ver. 1.3
Repf = 0                              ' clear repeat flag
lastaverf = 0
IF Htype > 0 THEN                     ' *** semi and manual holder
	  GOSUB diskeys16                    ' ver. 1.6
	  IF RepModef = 1 THEN
		  GOSUB dispselect
		  Clearbuff
		  PRINT #1, "@";
		  GOSUB receive
		  IF MID$(a$, 1, 6) = "Repeat" THEN
			  Repf = 1
			  Nrep = 1                            ' current N. of repetition
		  END IF
	  END IF
	  Clearbuff
	  cmd$ = "g"
	  IF simul THEN cmd$ = RIGHT$(STR$(P), 1)           ' !!!!!
	  PRINT #1, cmd$;                                   ' *** start meas
				COLOR 14
				LOCATE 2, 52
				IF Repf = 1 THEN
					PRINT "  <Esc> to finish repetition ";
				ELSE
					PRINT "              <Esc>  to stop ";
				END IF
				COLOR 7
	  LOCATE 1, 52
	  PRINT SPACE$(28);
	  LOCATE 1, 52
	  COLOR 15
	  PRINT "           *** P"; RIGHT$(STR$(P), 1); " IS RUNNING "
	  COLOR 7
	  IF P = 1 THEN
		  KEY 8, "      "
		  KEY(8) OFF
	  END IF
	  DO
		  GOSUB diesc
		  GOSUB measpos
		  IF escf OR nodataf THEN GOTO escerr
	  LOOP WHILE Repf = 1
		LOCATE 2, 52
		COLOR 15
		PRINT "        *** Setting Position "
	  cmd$ = "i"
	  PRINT #1, cmd$;
	  IF NOT simul THEN GOSUB receive
	  IF escf THEN RETURN

	  IF Htype = 2 AND mmode > 0 AND P > 1 AND P < 6 THEN
		  cmd$ = "o"
		  IF mmode = 1 AND P = 4 THEN GOTO savetst
		  LOCATE 2, 52
		  COLOR 15
		  PRINT "        *** Setting Position "
		  PRINT #1, cmd$;
		  IF NOT simul THEN GOSUB receive
		  IF escf THEN RETURN
	  END IF
savetst:
	  GOSUB enkeys16                ' ver. 1.6
	  IF retf = 7 THEN
		  KEY 8, " Save "
		  KEY(8) ON
		  ON KEY(8) GOSUB ak8
	  END IF
	  RETURN

ELSE                                                   ' *** auto holder

  KEY 1, "      ": KEY(1) OFF
  KEY 9, "      ": KEY(9) OFF
  KEY 10, "      ": KEY(10) OFF         ' ver. 1.3
  Clearbuff
  cmd$ = "r"
  PRINT #1, cmd$;
		LOCATE 2, 52
		COLOR 15
		PRINT "      *** Checking Rest Pos. "
		Pause .5
  GOSUB receive
  IF escf OR errf THEN GOTO escerr
  cmd$ = "y"                                       ' meas routine
  GOSUB diesc
  PRINT #1, cmd$;
		LOCATE 2, 52
		COLOR 15
		PRINT "      *** Checking Init Pos. "
		Pause .5
  GOSUB receive
  IF escf OR errf THEN GOTO escerr
			IF holpos <> 1 THEN
				LOCATE 1, 52
				BEEP
				COLOR 12
				PRINT "   --> Reset holder position"
				LOCATE 2, 52
				COLOR 14
				PRINT "            <CR> to continue"
				COLOR 7
				DO
				  q$ = INPUT$(1)
				LOOP WHILE q$ <> CHR$(13)
			END IF
  holpos = 1

  FOR P = 1 TO 3
	  IF RepModef = 1 THEN
		  GOSUB dispselect
		  PRINT #1, "@";
		  GOSUB receive
		  Repf = 1
		  Nrep = 1                            ' current N. of repetition
	  END IF
	  cmd$ = "g"
	  IF simul THEN cmd$ = RIGHT$(STR$(P), 1)           ' !!!!!
	  PRINT #1, cmd$;                   '*** start meas
				COLOR 14
				LOCATE 2, 52
				IF Repf = 1 THEN
					PRINT "  <Esc> to finish repetition ";
				ELSE
					PRINT "              <Esc>  to stop ";
				END IF
				COLOR 7
	  LOCATE 1, 52
	  PRINT SPACE$(28);
	  LOCATE 1, 52
	  COLOR 15
	  PRINT "           *** P"; RIGHT$(STR$(P), 1); " IS RUNNING "
	  COLOR 7
	  IF P = 1 THEN
		  KEY 8, "      "
		  KEY(8) OFF
	  END IF

	  DO
		  GOSUB diesc
		  GOSUB measpos
		  IF escf OR nodataf THEN GOTO escerr
	  LOOP WHILE Repf = 1

	  
	  LOCATE 2, 52
	  PRINT SPACE$(28);
	  cmd$ = "i"
	  PRINT #1, cmd$;
		LOCATE 2, 52
		COLOR 15
		PRINT "       *** Setting Main Pos."
		COLOR 7
	  IF NOT simul THEN GOSUB receive
	  IF escf OR errf THEN GOTO escerr
		 Pause .5
	  cmd$ = "w"
	  PRINT #1, cmd$;
		 LOCATE 2, 52
		 COLOR 15
		 PRINT "       *** Setting Work Pos."
		 COLOR 7
	  GOSUB receive
	  IF escf OR errf THEN GOTO escerr
		 Pause .5
	  cmd$ = "t"
	  GOSUB diesc
	  PRINT #1, cmd$;
		 LOCATE 2, 52
		 COLOR 15
		 PRINT "       *** Setting Next Pos."
		 COLOR 7
	  GOSUB receive
	  IF escf OR errf THEN GOTO escerr
		 Pause .5
	  holpos = holpos + 1
	  IF holpos = 4 THEN holpos = 1
	  GOSUB diesc
	  cmd$ = "r"
	  PRINT #1, cmd$;
		 LOCATE 2, 52
		 COLOR 15
		 PRINT "       *** Setting Rest Pos."
		 COLOR 7
	  GOSUB receive
	  IF escf OR errf THEN GOTO escerr
  NEXT P
  KEY 1, " Start": KEY(1) ON
  IF retf = 7 THEN
	  KEY 8, " Save "
	  KEY(8) ON
	  ON KEY(8) GOSUB ak8
  END IF
RETURN
END IF

escerr:
IF Htype = 0 AND escf AND holpos <> 1 AND NOT errf THEN

		Clearbuff
		cmd$ = "i"              ' esc in meas/corr    ver.1.3
		GOSUB diesc
		PRINT #1, cmd$;
		  LOCATE 2, 52
		  COLOR 15
		  PRINT "       *** Setting Main Pos."
		  Pause .5
		GOSUB receive
		IF errf THEN GOTO kon
		
		Clearbuff
		cmd$ = "w"
		PRINT #1, cmd$;
		  LOCATE 2, 52
		  COLOR 15
		  PRINT "       *** Setting Work Pos."
		  Pause .5
		  COLOR 7
		GOSUB receive
		IF errf THEN GOTO kon
		  LOCATE 1, 52
		  PRINT SPACE$(28);
		  Pause .5
		Clearbuff
		cmd$ = "y"              ' esc in meas/corr
		GOSUB diesc
		PRINT #1, cmd$;
		  LOCATE 2, 52
		  COLOR 15
		  PRINT "       *** Setting Init Pos."
		  Pause .5
		GOSUB receive
		IF errf THEN GOTO kon
		holpos = 1
		  LOCATE 1, 52
		  PRINT SPACE$(28);
		  Pause .5
		Clearbuff
		cmd$ = "r"
		PRINT #1, cmd$;
		  LOCATE 2, 52
		  COLOR 15
		  PRINT "       *** Setting Rest Pos."
		  Pause .5
		  COLOR 7
		GOSUB receive
END IF
kon:
IF errf THEN GOSUB enesc
RETURN



mpj:
	  IF Rm <> 9999 THEN
		  KEY 7, "      ": KEY(7) OFF
		  KEY 9, "      ": KEY(9) OFF
		  GOTO mp1
	  END IF
	  LOCATE 1, 52
	  PRINT "     --> START POSITION Nr.1";
	  BEEP
	  RETURN


'***** measuring mode 2-4-6 : mmode <0,1,2>
ak7:

mmode = mmode + 1
IF mmode = 3 THEN mmode = 0
SELECT CASE mmode
CASE 0
	  KEY 3, "      ": KEY(3) OFF
	  KEY 4, "      ": KEY(4) OFF
	  KEY 5, "      ": KEY(5) OFF
	  KEY 6, "      ": KEY(6) OFF

CASE 1
	  KEY 3, "  P3  ": KEY(3) ON
	  KEY 4, "  P4  ": KEY(4) ON

CASE 2
	  KEY 5, "  P5  ": KEY(5) ON
	  KEY 6, "  P6  ": KEY(6) ON

END SELECT
RETURN


ak8:

IF T(1) = 9999 THEN
	Clearbuff
	IF simul THEN P = 8            ' !!!!!
	cmd$ = "@"
	PRINT #1, cmd$;
	GOSUB receive
	IF MID$(a$, 1, 6) = "Repeat" THEN
		RepModef = 1                        ' repeat mode flag
		Repf = 1
		Nrep = 1                            ' current N. of repetition
	END IF
	GOSUB dispselect
	RETURN
END IF

IF retf = 7 THEN
	GOSUB writedatafile
	GOSUB printscreen
	retf = 1
	RepModef = 0
	Repf = 0
	measopen = 0
	specrout = 0
ELSE
	LOCATE 1, 52
	PRINT "   ###  MISSING POSITION! "
	BEEP
END IF
RETURN


'***** auto range
'smk5:
' Clearbuff
' cmd$ = "a"
' PRINT #1, cmd$;
'RETURN


'***** short-normal-long time of measurement
ak9:

mtime = mtime + 1
IF mtime = 3 THEN mtime = 0
mtime$ = "-??"
Clearbuff
SELECT CASE mtime
CASE 0                         ' normal
  cmd$ = CHR$(26)
  PRINT #1, cmd$;
  GOSUB receive
  IF escf THEN RETURN
  IF LEFT$(a$, 6) = "Normal" THEN mtime$ = "   "
  NR = 880

CASE 1                         ' short
  cmd$ = CHR$(25)
  PRINT #1, cmd$;
  GOSUB receive
  IF escf THEN RETURN
  IF LEFT$(a$, 5) = "Short" THEN mtime$ = "-ST"
  NR = 440

CASE 2                         ' long
  cmd$ = CHR$(24)
  PRINT #1, cmd$;
  GOSUB receive
  IF escf THEN RETURN
  IF LEFT$(a$, 4) = "Long" THEN mtime$ = "-LT"
  NR = 1760
END SELECT
GOSUB dispselect
RETURN


ak10:
zpet12:
LOCATE 1, 52
PRINT SPACE$(28);
LOCATE 23, 52
PRINT "    --> Confirm Kill [Y/N] ? "
BEEP
q$ = INPUT$(1)                        '******
'q$ = "Y"                               '******
IF UCASE$(q$) = "Y" THEN
	CLS
	retf = 1
	IF simul THEN P = 9                        ' !!!!!
	GOSUB mainpage
ELSEIF UCASE$(q$) <> "N" THEN
	GOTO zpet12
ELSE
	LOCATE 23, 52
	PRINT SPACE$(28);
END IF
RETURN


'**************************************** r keys ****************************

 KEY 1, " E-4  ": ON KEY(1) GOSUB rg1
 KEY 2, " E-3  ": ON KEY(2) GOSUB rg2
 KEY 3, " E-2  ": ON KEY(3) GOSUB rg3
 KEY 4, " E-1  ": ON KEY(4) GOSUB rg4
 KEY 5, " E+0  ": ON KEY(5) GOSUB rg5
 KEY 6, " E+1  ": ON KEY(6) GOSUB rg6
 KEY 7, " E+2  ": ON KEY(7) GOSUB rg7
 KEY 8, "      ": ON KEY(8) GOSUB rg8: KEY(8) ON
 KEY 9, "      ": ON KEY(9) GOSUB rg9: KEY(9) ON
 KEY 10, " QUIT ": ON KEY(10) GOSUB rg10: KEY(10) ON
 RETURN

rg1: PRINT #1, "0"; : retf = 1: RETURN
rg2: PRINT #1, "1"; : retf = 1: RETURN
rg3: PRINT #1, "2"; : retf = 1: RETURN
rg4: PRINT #1, "3"; : retf = 1: RETURN
rg5: PRINT #1, "4"; : retf = 1: RETURN
rg6: PRINT #1, "5"; : retf = 1: RETURN
rg7: PRINT #1, "6"; : retf = 1: RETURN

rg8:
RETURN

rg9:
RETURN

rg10:
RETURN


'***** measurement in current position
measpos:

IF RepModef = 1 AND Nrep = MaxRep THEN GOSUB exitrepeat
GOSUB receive
IF escf OR nodataf THEN RETURN
GOSUB rotgain
GOSUB subhol
GOSUB changesgn
IF Nrep <> 0 THEN GOSUB lastaver
GOSUB zobraz
T(2 * P - 1) = xu
T(2 * P) = yu
Top(2 * P - 1) = xu
Top(2 * P) = yu
Rc = rg - 5
measopen = 0
FOR jj = 2 TO 6
	IF T(2 * jj) <> 9999 THEN measopen = 1
NEXT jj
IF measopen = 0 AND P = 1 AND lastaverf = 0 THEN Rm = Rc
IF Rc > Rm THEN GOSUB opravrange
IF measopen = 1 AND P = 1 AND lastaverf = 0 AND Rc <> Rm THEN
	FOR jj = 3 TO 12
	  T(jj) = 9999
	NEXT jj
	Rm = Rc
	retf = 0
	GOSUB dispheader
END IF
fak = 10 ^ (Rc - Rm)
T(2 * P - 1) = T(2 * P - 1) * fak
T(2 * P) = T(2 * P) * fak
Top(2 * P - 1) = Top(2 * P - 1) * fak
Top(2 * P) = Top(2 * P) * fak
GOSUB eval
GOSUB dispdata
IF retf = 7 THEN GOSUB endmeas
RETURN

'***** holder substraction
subhol:
IF Htype = 2 THEN
	xt = xt - Hm1
	yt = yt - Hm2
ELSE
	xt = xt - Ha(2 * P - 1)
	yt = yt - Ha(2 * P)
END IF
RETURN

simuldata:                                  ' !!!!! for manual holder only
	 SELECT CASE P                           '       hs, phase -37, spec.K2
		 CASE 0
		a$ = "GR4MS*STOP|Q" + CHR$(13)                 ' <ESC>
		 CASE 1
		a$ = "GR4M* +0184 +1367 A4| 0.1%" + CHR$(13)
		 CASE 2
		a$ = "GR5M* -0224 -0049 A5| 0.1%" + CHR$(13)
		 CASE 3
		a$ = "GR4M* -0183 -1363 A4| 0.1%" + CHR$(13)
		 CASE 4
		a$ = "GR5M* +0225 +0050 A5| 0.3%" + CHR$(13)
		 CASE 5
		a$ = "GR5M* -0204 +0104 A5| 0.1%" + CHR$(13)
		 CASE 6
		a$ = "GR5M* +0206 -0102 A5| 1.2%" + CHR$(13)
		 CASE 7
		a$ = "GR4M* -0505 +0642 A4| 0.1%" + CHR$(13) ' Cal.ET: 7.13 A/m 8 ccm
		 CASE 8
		a$ = "Repeat mode" + CHR$(13)                ' cmd @
		 CASE 9
		a$ = "Auto range " + CHR$(1) + CHR$(13)      ' cmd "a"  and "KILL"
	 END SELECT
	 IF P > 0 THEN Pause 1
RETURN


'*****  wait for a$ response with timeout and getdata if posible
receive:

IF cmd$ = "g" THEN
	 IF Speed = 0 THEN
		 timeout = 15 + NR / 87 / (Speed + 1)
	 ELSE
		 timeout = 15 + NR / 16 / (Speed + 1)
	 END IF
ELSEIF cmd$ = "i" THEN timeout = 60
ELSEIF cmd$ = "o" THEN timeout = 10             ' ver. 1.3
ELSEIF cmd$ = "y" THEN timeout = 25
ELSEIF cmd$ = "t" THEN timeout = 10
ELSEIF cmd$ = "r" OR cmd$ = "w" THEN timeout = 7
ELSE timeout = 3
END IF

a$ = ""
a1$ = ""
IF simul THEN GOSUB simuldata      ' !!!!! del "gosub" for simul by other PC
errf = 0
escf = 0
star1 = TIMER
IF cmd$ = "g" THEN GOSUB enesc             ' esc enable in receive loop only
DO
IF escf OR simul THEN EXIT DO     ' !!!!! del "or simul" for simul by other PC
		IF LOC(1) > 0 THEN
			ON ERROR GOTO repeatact
			'''IF a1$ = "*" AND actx = 0 THEN act = 5 / 0
			'''IF a1$ = "*" AND actx = 0 AND Nrep = 5 THEN act = 5 / 0
			a1$ = INPUT$(1, #1)
			ON ERROR GOTO 0
			a$ = a$ + a1$
			IF LEN(a$) = 1 AND cmd$ = "g" THEN
				COLOR 7
				LOCATE 1, 52
				PRINT "  ..........................";
			END IF
			IF a1$ = CHR$(13) THEN EXIT DO
		END IF
		
		IF cmd$ = "g" THEN
		 '''IF LEN(a$) = 25 THEN a$ = a$ + "EEE"
			IF LEN(a$) > 27 THEN
				a$ = RIGHT$(a$, 27)
				LOCATE 1, 52 + 1
				COLOR 12
				PRINT a$;                ' disp instrument message
				COLOR 7
			ELSE
				LOCATE 1, 52 + LEN(a$) + 1
				COLOR 10
				PRINT a1$;                ' disp partial instrument message
				COLOR 7
			END IF
		END IF
		wt = TIMER - star1
		IF wt > timeout THEN
			COLOR 12
			LOCATE 1, 52
			a$ = "     ### No message received"
			PRINT a$;
			LOCATE 23, 52
			PRINT SPACE$(28);                       ' clear action message
			COLOR 7
			SOUND 1000, 5
			escf = -1
		END IF
LOOP WHILE -1
GOSUB diesc                                      ' esc disable
IF escf THEN RETURN
IF INSTR(a$, "#") > 0 THEN
	errf = -1
END IF
IF INSTR(a$, "|") > 0 THEN
	Repf = 0
END IF

a$ = LEFT$(a$, LEN(a$) - 1)
IF a$ = "Ok" THEN
	LOCATE 2, 52                            ' disp message - instrument format
	PRINT SPACE$(28);
	LOCATE 2, 80 - LEN(a$)
	COLOR 10
	PRINT a$
	Pause .7
ELSE
	LOCATE 1, 52                            ' disp message - instrument format
	PRINT SPACE$(28);
	IF LEN(a$) > 28 THEN a$ = RIGHT$(a$, 28)
	LOCATE 1, 80 - LEN(a$)
	COLOR 10
	PRINT a$;                               ' disp message in instrument format
END IF

COLOR 7
LOCATE 23, 52
PRINT SPACE$(28);                          ' clear action message
nodataf = 0
pas = INSTR(a$, "*")
IF pas = 0 OR MID$(a$, pas + 1, 1) <> " " THEN
	nodataf = -1
	IF cmd$ = "g" AND MID$(a$, 5, 2) <> "#E" THEN
		LOCATE 2, 52
		COLOR 12
		IF LEFT$(a$, 1) <> "G" AND Nrep < 2 THEN   ' ver. 1.5
			PRINT "  ## Illegal message format!"
			GOSUB enesc                             ' ver. 1.6
		END IF
		COLOR 7
	END IF
	RETURN
END IF
rg = VAL(MID$(a$, pas + 15, 1))
x = VAL(MID$(a$, pas + 2, 5)) * 10 ^ (rg - 6)
y = VAL(MID$(a$, pas + 8, 5)) * 10 ^ (rg - 6)
IF cmd$ = "g" AND LEFT$(a$, 1) <> "G" AND Nrep < 2 THEN  ' ver. 1.5
	LOCATE 2, 52
	COLOR 12
	PRINT "  ## Illegal message format!"
	SOUND 1000, 20
	GOSUB enesc                            ' ver. 1.6
	Pause 5
	COLOR 7
END IF
actx = 0
RETURN

repeatact:
'LOCATE 1, 52
'COLOR 12
'PRINT "         ## Serial Com Error"
'SOUND 1000, 20
Pause 1.5
Clearbuff
IF cmd$ = "g" THEN
	actx = 1
	cmd$ = "?"
	IF Repf = 1 THEN
		GOSUB Esc
	END IF
ELSE
	cmd$ = "a"
END IF
PRINT #1, cmd$;
'LOCATE 2, 52
'PRINT "                            "
RESUME receive


'***** correction for range
opravrange:

ofak = 10 ^ (Rm - Rc)
FOR oprp = 1 TO 6
	IF T(2 * oprp) <> 9999 AND oprp <> P THEN
		T(2 * oprp - 1) = T(2 * oprp - 1) * ofak
		T(2 * oprp) = T(2 * oprp) * ofak
	END IF
NEXT oprp
Rm = Rc:
GOSUB dispdata
RETURN


'manualrange:
'   Clearbuff
'   PRINT #1, cmd$;
'   GOSUB receive
'RETURN


'***** rotate coordinate system by phase
rotgain:
	 phs = -fi / deg
	 xt = x * COS(phs) + y * SIN(phs)
	 yt = -x * SIN(phs) + y * COS(phs)
	 xt = xt * gnom / Vact
	 yt = yt * gnom / Vact
	 amp = SQR(xt * xt + yt * yt)
RETURN


'***** correction for signum in current position
changesgn:
	 IF Htype = 2 THEN                   ' manual holder
	IF P > 2 AND P < 6 THEN
		xt = -xt
		yt = -yt
	END IF
	 ELSE
		 IF P = 2 THEN yt = -yt
		 IF P = 3 THEN xt = -xt
		 IF P > 1 THEN
	  pom = yt
	  yt = xt
	  xt = pom
		 END IF
	 END IF
RETURN


'***** transfer to display format
zobraz:
	xu = xt * 10 ^ (5 - rg)
	yu = yt * 10 ^ (5 - rg)
	DO WHILE ABS(xu) > 19.99 OR ABS(yu) > 19.99
		xu = xu / 10
		yu = yu / 10
		rg = rg + 1
	LOOP
	m = SQR(xu * xu + yu * yu)
	'LOCATE 1, 52
	'PRINT USING " P# +##.## +##.## +##.## E+#"; P; xu; yu; m; rg - 5
	'PRINT USING " P# +##.## +##.## E+#"; P; xu; yu; rg - 5
RETURN


'***** store configuration file
writeconfig:
OPEN "PARJR6.CFG" FOR OUTPUT AS #5
PRINT #5, USING "### &"; sport; TAB(30); RIGHT$(doc(2), LEN(doc(2)) - 5)
PRINT #5, USING "###  ###  ###  ### &"; IP1; IP2; IP3; IP4; TAB(30); RIGHT$(doc(3), LEN(doc(3)) - 5)
PRINT #5, USING "###.## &"; Vact; TAB(30); RIGHT$(doc(4), LEN(doc(4)) - 5)
PRINT #5, USING "###.##  ###.## &"; ETALON; Vetal; TAB(30); RIGHT$(doc(6), LEN(doc(6)) - 5)
PRINT #5, USING "### &"; Htype; TAB(30); RIGHT$(doc(7), LEN(doc(7)) - 5)
PRINT #5, USING "### &"; Stype; TAB(30); RIGHT$(doc7, LEN(doc7) - 5)
PRINT #5, USING "### &"; Speed; TAB(30); RIGHT$(doc(8), LEN(doc(8)) - 5)
PRINT #5, USING " ##.#####^^^^ ##.#####^^^^ &"; Gnomh; Fih; TAB(30); RIGHT$(doc(11), LEN(doc(11)) - 5)
PRINT #5, USING " ##.#####^^^^ ##.#####^^^^ &"; Gnoml; Fil; TAB(30); RIGHT$(doc(12), LEN(doc(12)) - 5)
PRINT #5, USING " ##.#####^^^^ ##.#####^^^^ &"; Hm1; Hm2; TAB(30); RIGHT$(doc(13), LEN(doc(13)) - 5)
PRINT #5, USING " ##.#####^^^^ ##.#####^^^^ &"; Ha(1); Ha(2); TAB(30); RIGHT$(doc(14), LEN(doc(14)) - 5)
PRINT #5, USING " ##.#####^^^^ ##.#####^^^^ &"; Ha(3); Ha(4); TAB(30); RIGHT$(doc(15), LEN(doc(15)) - 5)
PRINT #5, USING " ##.#####^^^^ ##.#####^^^^ &"; Ha(5); Ha(6); TAB(30); RIGHT$(doc(16), LEN(doc(16)) - 5)
CLOSE #5
RETURN


'***** read configuration file
readconfig:
ON ERROR GOTO noparam
OPEN "PARJR6.CFG" FOR INPUT AS #5
INPUT #5, sport, dum$
	 IF sport <> 1 AND sport <> 2 THEN sport = 1
INPUT #5, z(1), z(2), z(3), z(4), dum$
	 GOSUB testorparam
INPUT #5, Vact, dum$
INPUT #5, ETALON, Vetal, dum$
INPUT #5, Htype, dum$
	 IF Htype < 0 OR Htype > 2 THEN Htype = 2
	 IF Htype = 1 THEN automf = -1
INPUT #5, Stype, dum$
INPUT #5, Speed, dum$
INPUT #5, Gnomh, Fih, dum$
INPUT #5, Gnoml, Fil, dum$
SELECT CASE Speed
  CASE 0
	  fi = Fih
	  gnom = Gnomh
  CASE 1
	  fi = Fil
	  gnom = Gnoml
END SELECT
INPUT #5, Hm1, Hm2, dum$
FOR cp = 1 TO 3
	 INPUT #5, Ha(2 * cp - 1), Ha(2 * cp), dum$
NEXT cp
ON ERROR GOTO 0
CLOSE #5
RETURN


testprot:
	 ON ERROR GOTO prot
	 OPEN path$ + "write.dis" FOR OUTPUT AS #7
	 ON ERROR GOTO 0
	 CLOSE #7
	 KILL path$ + "write.dis"
RETURN

prot:
	 PRINT "### Opening file error"
	 PRINT
	 BEEP
	 PRINT "*** Diskette may be write protect or full"
	 PRINT "--> Check the diskette and press any key to continue"
	 q$ = INPUT$(1)
RESUME Selectpath

missing:
RESUME nofile

invalidpath:
	 LOCATE ilinz, 1
	 PRINT SPACE$(79);
	 LOCATE ilinz, 1
	 PRINT "*** Directory does not exist"
	 BEEP
	 PRINT
	 PRINT "--> Do you wish to create it [Y/N] ?"
zpet9:
	 q$ = INPUT$(1): q$ = UCASE$(q$)
	 IF q$ = "Y" THEN
		 path$ = LEFT$(path$, LEN(path$) - 1)
		 MKDIR path$
	 ELSEIF q$ <> "N" THEN
		 BEEP
		 GOTO zpet9
	 END IF
	 CLS
RESUME Selectpath

invalidgeopath:
	 PRINT "### No geological file in  "; geopath$
	 BEEP
	 PRINT
	 PRINT " --> Press any key to continue"
	 q$ = INPUT$(1)
	 CLS
	 PRINT "*** Selection of the geological file "
	 PRINT
RESUME ingeo

notexist:
	 VIEW PRINT 20 TO 24
	 CLS
	 VIEW PRINT 1 TO 24
	 LOCATE 20, 1
	 PRINT " ### COM" + RIGHT$(STR$(sport), 1) + " is not available on this computer"
	 BEEP
	 IF sport = 1 THEN sport = 2 ELSE sport = 1
	 PRINT " *** Serial Port will be changed to COM"; RIGHT$(STR$(sport), 1)
	 LOCATE 23, 1
	 PRINT " --> Press any key to continue"
	 q$ = INPUT$(1)
	 GOSUB mainpage
RESUME serport


noparam:
	PRINT "### Parameter Configuration File  PARJR6.CFG is missing or damaged"
	BEEP
	PRINT
	PRINT "*** You can copy the file to current directory or create a new one"
	PRINT "*** If you create a new file you must set some parameters and"
	PRINT "    CALIBRATE the instrument"
	PRINT
	PRINT "--> Create new parameter file or Exit program [C/E] ? "
zpet8:
	q$ = INPUT$(1)
	IF UCASE$(q$) = "C" THEN
		 PRINT
		 PRINT "*** New Parameter File has been created"
		 PRINT
		 PRINT "--> Set the appropriate parameters"
		 PRINT "--> Execute Calibration"
		 PRINT
		 PRINT "    Press any key to continue"
		 q$ = INPUT$(1)
		 sport = 1
		 gnom = 1!
		 Gnomh = gnom
		 Gnoml = gnom
		 Htype = 2
		 GOSUB writeconfig
		 RESUME Selectpath
	ELSEIF UCASE$(q$) = "E" THEN
		 GOTO mk10a
	ELSE
		 BEEP
		 GOTO zpet8
	END IF
RETURN


exitrepeat:
	PRINT #1, "x";
	Repf = 0
	BEEP
RETURN

																		 'last three average
lastaver:
	xurep = xt * 10 ^ (4 - rg)
	yurep = yt * 10 ^ (4 - rg)
	LOCATE Nrep + 3, 56
	IF Htype < 2 THEN LOCATE Nrep + 5, 56
	PRINT USING "##. P# +##.## +##.## E+#"; Nrep; P; xurep; yurep; rg - 4

	a1rep(Nrep) = xt
	a2rep(Nrep) = yt
	Nrep = Nrep + 1
	IF Nrep < 4 THEN RETURN
	IF Nrep < 5 THEN M1 = 1 ELSE M1 = Nrep - 3
	AS1 = 0
	AS2 = 0
	FOR i = M1 TO Nrep - 1
		AS1 = AS1 + a1rep(i)
		AS2 = AS2 + a2rep(i)
	NEXT i
	xt = AS1 / (Nrep - M1)
	yt = AS2 / (Nrep - M1)
	lastaverf = 1
RETURN


dispselect:

  LOCATE 1, 1
  PRINT SPACE$(60);
  LOCATE 1, 1
  COLOR 15
  'COLOR 23
  IF Htype = 2 THEN
	  PRINT "Manual holder    ";
  ELSEIF Htype = 1 THEN
	  PRINT "Semi-Auto holder ";
  ELSE
	  PRINT "Automatic holder ";
  END IF

  LOCATE 1, 20
  IF Speed = 0 THEN PRINT "High speed" ELSE PRINT "Low speed"

  IF RepModef = 1 THEN
	  FOR i = 4 TO 22
		 LOCATE i, 52
		 PRINT "                             ";
	  NEXT i
	  LOCATE 1, 33
	  COLOR 15
	  PRINT "Repeat mode"
  END IF

  LOCATE 2, 1
  PRINT SPACE$(50);
  LOCATE 2, 1
  PRINT USING "Vol:##.##"; Vact;
  IF Htype < 2 THEN
	  IF Stype = 0 THEN PRINT " Cylin. ";  ELSE PRINT " Cubic ";
  END IF
  LOCATE 2, 20
  IF mtime$ = "   " THEN
	 PRINT "          ";
  ELSEIF mtime$ = "-LT" THEN
	 PRINT "Long time ";
  ELSEIF mtime$ = "-ST" THEN
	 PRINT "Short time";
  END IF
  LOCATE 2, 33

COLOR 7
RETURN

keysblankoff:
	FOR i = 1 TO 10
		KEY i, "     "
		KEY(i) OFF
	NEXT i
RETURN

diskeys16:
	 FOR i = 1 TO 6
		KEY(i) OFF
	 NEXT i
RETURN

enkeys16:
	 FOR i = 1 TO 6
		KEY(i) ON
	 NEXT i
RETURN
	 
keysoff:
	 FOR i = 1 TO 10
		KEY i, ""
		KEY(i) OFF
	 NEXT i
	KEY OFF
RETURN

keyson:
	FOR i = 1 TO 10
	  KEY(i) ON
	NEXT i
	KEY ON
RETURN



'***** subroutine specimen - inputting data (read .GED file)
specimen:

  GOSUB keysoff
  Rm = 9999            ' blank measuring range
  specrout = -1
  FOR i = 1 TO 12      ' blank data
	  T(i) = 9999
  NEXT i
  CLS
  Mode = 0
  PRINT "*** Input of specimen data "
  PRINT
  

specname:
  PRINT "Specimen name (max.10 chars) : "; Spec$; " ";
  INPUT qq$
  qq$ = UCASE$(qq$)
  IF qq$ <> "" THEN
		 Spec$ = qq$
		 namk$ = qq$
  END IF
  IF LEN(Spec$) < 1 OR LEN(Spec$) > 10 THEN
	  BEEP
	  LOCATE CSRLIN - 1, 1
	  PRINT SPACE$(79);
	  LOCATE CSRLIN, 1
	  GOTO specname
  END IF
  LOCATE CSRLIN - 1, 1
  PRINT SPACE$(79);
  LOCATE CSRLIN, 1
  PRINT "SPECIMEN NAME                   "; Spec$

specnote:
  PRINT "Demagnet. note (max.8 chars) : "; Dem$; " ? ";
  LINE INPUT qqq$
  qqq$ = UCASE$(qqq$)
  IF qqq$ <> "" THEN Dem$ = qqq$
  IF LEFT$(qqq$, 1) = " " THEN Dem$ = ""
  IF LEN(Dem$) > 8 THEN
	  BEEP
	  LOCATE CSRLIN - 1, 1
	  PRINT SPACE$(79);
	  LOCATE CSRLIN, 1
	  GOTO specnote
  END IF
  LOCATE CSRLIN - 1, 1
  PRINT SPACE$(79);
  LOCATE CSRLIN, 1
  PRINT "DEMAGNETIZATION NOTE            "; Dem$

  findf = 0
  IF entermode = 1 THEN GOSUB searchgd            ' try to find in .GED file

  IF entermode > 1 OR NOT findf THEN              ' manual input block
	  IF nonor THEN
		  IpfiA = 0
		  IpsiA = 0
		  Ipfi = IpfiA
		  Ipsi = IpsiA
		  GOTO anychg
	  END IF
	  PRINT
	  PRINT "*** Enter comma separated SPECIMEN ANGLES or <CR>"
	  PRINT
	  IF IP3 = 3 THEN
		  PRINT "Azimuth of Strike,";
		  IF IP2 = 90 THEN
			  PRINT " Plunge ";
		  ELSE
			  PRINT " Dip ";
		  END IF
	  ELSEIF IP3 = 6 THEN
		  PRINT "Azimuth of negative Dip,";
		  IF IP2 = 90 THEN
			  PRINT " Plunge ";
		  ELSE
			  PRINT " Dip ";
		  END IF
	  ELSEIF IP3 = 12 THEN
		  PRINT "Azimuth of Dip,";
		  IF IP2 = 90 THEN
			  PRINT " Plunge ";
		  ELSE
			  PRINT " Dip ";
		  END IF
	  END IF
	  GOSUB inpangl
	  IF entermode < 3 AND in1 = 0 AND in2 = 0 THEN in1 = 1
	  IpfiA = in1
	  IpsiA = in2
	  Ipfi = IpfiA
	  Ipsi = IpsiA

	  IrhoA = 0: : Irho = 0
	  Imu = 0
	  ItauA = 0: Itau = 0
	  IsigA = 0: Isig = 0

	  PRINT
	  IF Tectcorr > 0 THEN
		  PRINT "*** Enter comma separated TECTONIC ANGLES or <CR>"
		  PRINT
		  IF IP4 = 0 THEN
			  PRINT "FOLIATION: Azimuth of Dip, Dip ";
		  ELSE
			  PRINT "FOLIATION: Azimuth of Strike, Dip ";
		  END IF

		  GOSUB inpangl
		  IrhoA = in1
		  Imu = in2
		  Irho = IrhoA

		  IF IrhoA = 0 AND Imu = 0 THEN GOTO anychg
		  IF Tectcorr > 1 THEN
			  PRINT "LINEATION: Trend, Plunge ";
			  GOSUB inpangl
			  ItauA = in1
			  IsigA = in2
			  Itau = ItauA
			  Isig = IsigA
		  END IF
	  END IF
anychg:
	  PRINT
	  PRINT "Any changes [Y/N or <CR>] ?";
	  q$ = INPUT$(1)
	  q$ = UCASE$(q$)
	  PRINT q$
	  IF q$ = "Y" THEN GOTO specimen
  END IF                                       ' end of manual input block

  CLS
  GOSUB coran
  GOSUB dispheader
  GOSUB keyson
RETURN

inpangl:
  PRINT " : ";
  LINE INPUT inp$
  linp = LEN(inp$)
  PC = INSTR(inp$, ",")
  IF linp = 0 THEN
	  in1 = 0
	  in2 = 0
  ELSEIF PC = 0 THEN
	  in1 = VAL(inp$)
	  in2 = 0
  ELSE
	  in1 = VAL(MID$(inp$, 1, PC - 1))
	  in2 = VAL(MID$(inp$, PC + 1, linp - PC))
  END IF
RETURN


'***** SUB endmeas
endmeas:
 GOSUB dispaver
 XV = x: YV = y: ZV = z
 GOSUB DeclIncl
 DECLs = Decl: INCLs = Incl
 x1 = x: x2 = y: X3 = z
 c1 = COS(F1): c2 = COS(F2): c3 = COS(F3)
 s1 = SIN(F1): s2 = SIN(F2): s3 = SIN(F3)
 Xg = x1 * (c1 * c2 * c3 - s1 * s3) + x2 * (-c1 * c2 * s3 - s1 * c3) + X3 * (-c1 * s2)
 Yg = x1 * (s1 * c2 * c3 + c1 * s3) + x2 * (-s1 * c2 * s3 + c1 * c3) + X3 * (-s1 * s2)
 Zg = x1 * (s2 * c3) + x2 * (-s2 * s3) + X3 * c2
 XV = Xg: YV = Yg: ZV = Zg
 GOSUB DeclIncl
 DECLg = Decl: INCLg = Incl
 IF Tectcorr > 0 AND (IrhoA <> 0 OR Imu <> 0) THEN
	 w = Irho + 270                                   ' fictive lineation
	 Itau = w MOD 360
	 Isig = 0
	 GOSUB paleo                                      ' tilt correction
	 XV = XP: YV = YP: ZV = ZP
	 GOSUB DeclIncl
	 DECLpt = Decl: INCLpt = Incl
	 Itau = ItauA
	 Isig = IsigA
	 IF Tectcorr = 2 AND (ItauA <> 0 OR IsigA <> 0) THEN
		 GOSUB paleo                                   ' full correction
		 XV = XP: YV = YP: ZV = ZP
		 GOSUB DeclIncl
		 DECLpf = Decl: INCLpf = Incl
	 END IF
END IF
 GOSUB dispresults
 RETURN


'***** write to data file .JR6
writedatafile:

 PRINT #2, USING "\        \"; Spec$;
 PRINT #2, USING "\      \"; Dem$;
 PRINT #2, USING "###.##"; INT(x * 100 + .5) / 100;
 PRINT #2, USING "###.##"; INT(y * 100 + .5) / 100;
 PRINT #2, USING "###.##"; INT(z * 100 + .5) / 100;
 PRINT #2, USING "####"; Rm; IpfiA; IpsiA; IrhoA; Imu; ItauA; Isig;  '-----
 PRINT #2, USING "###"; IP1; IP2; IP3; IP4;
 PRINT #2, USING "####"; Prec;
 PRINT #2, CHR$(13) + CHR$(10);
 Rec% = Rec% + 1
 PUT #2, Rec%
 CLOSE #2
 OPEN "R", #2, Filedata$, 82
 FIELD #2, 10 AS F1$, 8 AS F2$, 6 AS F3$, 6 AS F4$, 6 AS F5$, 4 AS F6$, 4 AS F7$, 4 AS F8$, 4 AS F9$, 4 AS F10$, 4 AS F11$, 4 AS F12$, 3 AS F13$, 3 AS F14$, 3 AS F15$, 3 AS F16$, 4 AS F17$, 2 AS F18$
RETURN


'***** display header on screen
dispheader:

 GOSUB dispselect
 locshift = 0
 IF Htype = 2 THEN
	 locshift = locshift - 2
	 IF allpos = 6 AND NOT nonor THEN locshift = locshift - 1
 END IF
 lp = 6 + locshift
 LOCATE lp, 1
 PRINT Spec$; " / "; Dem$; TAB(34);
 PRINT DATE$
 PRINT "-------------------------------------------"
 IF NOT nonor THEN
	 PRINT USING "SPEC.ANGLES                      ####  ####"; IpfiA; IpsiA
	 IF Tectcorr > 0 THEN PRINT USING "TECT.ANGLES          ####  ####  ####  ####"; IrhoA; Imu; ItauA; IsigA
	 PRINT "-------------------------------------------"
 END IF
 PRINT "           M(x)       M(y)       M(z)"
 PRINT "-------------------------------------------"
 IF nonor THEN lp = lp + 1 ELSE lp = lp + 4
 IF Tectcorr > 0 THEN lp = lp + 1
RETURN


'***** display data on screen
dispdata:

IF T(9) <> 9999 THEN
	allpos = 6
	CLS
	GOSUB dispheader
END IF

IF a$ = "Ok" THEN
	LOCATE 2, 52                            ' disp message - instrument format
	PRINT SPACE$(28);
	LOCATE 2, 80 - LEN(a$)
	COLOR 10
	PRINT a$
	Pause .7
ELSE
	LOCATE 1, 52                            ' disp message - instrument format
	PRINT SPACE$(28);
	IF LEN(a$) > 28 THEN a$ = RIGHT$(a$, 28)
	LOCATE 1, 80 - LEN(a$)
	COLOR 10
	PRINT a$;
END IF

COLOR 7
LOCATE lp, 1
PRINT USING "E+#"; Rm;
PRINT Speed$; mtime$;
FOR oprp = 1 TO allpos
	LOCATE lp + 1 + oprp, 1
	PRINT SPACE$(43)
NEXT oprp
'-------------------------------------------------------  manual holder
IF Htype = 2 THEN
 FOR oprp = 1 TO allpos
	  IF T(2 * oprp) <> 9999 THEN
	LOCATE lp + 1 + oprp, 1
	ON oprp GOTO Pos13, Pos24, Pos13, Pos24, Pos56, Pos56
Pos13:  PRINT USING "#     ######.##             ######.##"; oprp; Top(2 * oprp - 1); Top(2 * oprp)
	GOTO nx
Pos24:  PRINT USING "#                ######.##  ######.##"; oprp; Top(2 * oprp - 1); Top(2 * oprp)
	GOTO nx
Pos56:  PRINT USING "#     ######.##  ######.##           "; oprp; Top(2 * oprp - 1); Top(2 * oprp)
	GOTO nx
	  END IF
nx:
 NEXT oprp
'----------------------------------------------- auto and semiauto holder
ELSE
 FOR oprp = 1 TO 3
	  IF T(2 * oprp) <> 9999 THEN
	LOCATE lp + 1 + oprp, 1
	ON oprp GOTO Pos1, Pos2, Pos3
Pos1:   PRINT USING "#     ######.##             ######.##"; oprp; T(2 * oprp - 1); T(2 * oprp)
	GOTO nxa
Pos2:   IF Stype = 1 THEN
		T1o = T(1)
		T3o = T(3)
		T1n = .707 * (T1o + T3o)
		T3n = .707 * (-T1o + T3o)
		PRINT USING "#                ######.##  ######.##"; oprp; T3n; T(2 * oprp)
		LOCATE lp + 1 - 1 + oprp, 1
		PRINT USING "#     ######.##             ######.##"; 1; T1n; T(2)
	ELSE
		PRINT USING "#                ######.##  ######.##"; oprp; T(2 * oprp - 1); T(2 * oprp)
	END IF
	GOTO nxa
Pos3:   IF Stype = 1 THEN
		T5o = T(5)
		T6o = T(6)
		T5n = .707 * (T5o + T6o)
		T6n = .707 * (-T5o + T6o)
		PRINT USING "#     ######.##  ######.##           "; oprp; T5n; T6n
	ELSE
		PRINT USING "#     ######.##  ######.##           "; oprp; T(2 * oprp - 1); T(2 * oprp)
	END IF
	  END IF
nxa:
 NEXT oprp
END IF
RETURN


'***** display averages on screen
dispaver:

 IF T(5) = 9999 THEN
	 LOCATE lp + 4, 1
 ELSEIF T(7) = 9999 THEN
	 LOCATE lp + 5, 1
 ELSEIF T(9) = 9999 THEN
	 LOCATE lp + 6, 1
 ELSE
	 LOCATE lp + 8, 1
 END IF
 PRINT "-------------------------------------------"
 PRINT USING "Mean   #####.##   #####.##   #####.##      "; x; y; z
 PRINT "Modulus   ";
 PRINT ENG(Ampl * 10 ^ Rm, 4);
 PRINT TAB(22); "A/m  ";
 IF Alfa <> 999 THEN
	 PRINT USING "Prec###.#&"; Alfa * deg; CHR$(248);
	 PRINT USING " ###.#"; Prec;
	 PRINT "%"
 ELSE
	 PRINT ""
 END IF
RETURN


'***** display results on screen
dispresults:

 PRINT "-------------------------------------------"
 PRINT USING "SPEC.S. (Dec,Inc)   #######    #######"; DECLs; INCLs
 IF nonor THEN RETURN
 PRINT USING "GEOGR.S.            #######    #######"; DECLg; INCLg
 IF Tectcorr > 0 AND (IrhoA <> 0 OR Imu <> 0 OR ItauA <> 0 OR IsigA <> 0) THEN
	 PRINT USING "TILT Corr.          #######    #######"; DECLpt; INCLpt
 END IF
 IF Tectcorr = 2 AND (ItauA <> 0 OR IsigA <> 0) THEN
	 PRINT USING "FULL Corr.          #######    #######"; DECLpf; INCLpf
 END IF
 '''PRINT P1, Q1
RETURN




'***** print header to txt file
printheader:

 PRINT #3, Spec$; " / "; Dem$; TAB(34);
 PRINT #3, DATE$
 PRINT #3, "-------------------------------------------"
 IF NOT nonor THEN
	 PRINT #3, USING "SPEC.ANGLES                      ####  ####"; IpfiA; IpsiA
	 IF Tectcorr > 0 THEN
		 PRINT #3, USING "TECT.ANGLES          ####  ####  ####  ####"; IrhoA; Imu; ItauA; IsigA
	 END IF
	 PRINT #3, "-------------------------------------------"
 END IF

 PRINT #3, USING "E+#"; Rm;
 PRINT #3, Speed$; mtime$;
 PRINT #3, "   M(x)       M(y)       M(z)"
 PRINT #3, "-------------------------------------------"
RETURN


'***** print data to txt file
printdata:

'------------------------------------------------------ manual holder
IF Htype = 2 THEN
 FOR oprp = 1 TO allpos
  IF T(2 * oprp) <> 9999 THEN
	  ON oprp GOTO Pos13p, Pos24p, Pos13p, Pos24p, Pos56p, Pos56p
Pos13p: PRINT #3, USING "#     ######.##             ######.##"; oprp; Top(2 * oprp - 1); Top(2 * oprp)
	  GOTO nxp
Pos24p: PRINT #3, USING "#                ######.##  ######.##"; oprp; Top(2 * oprp - 1); Top(2 * oprp)
	  GOTO nxp
Pos56p: PRINT #3, USING "#     ######.##  ######.##           "; oprp; Top(2 * oprp - 1); Top(2 * oprp)
	  GOTO nxp
  END IF
nxp:
 NEXT oprp
'--------------------------------------------------------auto and semiauto
ELSE
 IF Stype = 0 THEN                                      'cylinder
	 T1n = T(1)
	 T3n = T(3)
	 T5n = T(5)
	 T6n = T(6)
 END IF
 FOR oprp = 1 TO 3
	ON oprp GOTO Pos1p, Pos2p, Pos3p
Pos1p:  PRINT #3, USING "#     ######.##             ######.##"; oprp; T1n; T(2)
	GOTO nxap
Pos2p:  PRINT #3, USING "#                ######.##  ######.##"; oprp; T3n; T(4)
	GOTO nxap
Pos3p:  PRINT #3, USING "#     ######.##  ######.##           "; oprp; T5n; T6n
nxap:
 NEXT oprp
END IF
RETURN


'***** print avarages to txt file
printaver:

 PRINT #3, "-------------------------------------------"
 PRINT #3, USING "MEAN   #####.##   #####.##   #####.## "; x; y; z
 PRINT #3, "Modulus   ";
 PRINT #3, ENG(Ampl * 10 ^ Rm, 4);
 PRINT #3, TAB(22); "A/m  ";
 IF Alfa <> 999 THEN
	 PRINT #3, USING "Prec###.#&"; Alfa * deg; CHR$(248);
	 PRINT #3, USING " ###.#"; Prec;
	 PRINT #3, "%"
 ELSE
	 PRINT #3, ""
 END IF
 PRINT #3, "-------------------------------------------"
RETURN


'***** print results to txt file
printresults:

 IF nonor THEN
	 PRINT #3, "                     DECL        INCL"
 ELSE
	 PRINT #3, USING "O.P: ## ## ## ##     DECL        INCL"; IP1; IP2; IP3; IP4
 END IF
 PRINT #3, "-------------------------------------------"
 PRINT #3, USING "SPEC. S.             ####        ####"; DECLs; INCLs
 IF NOT nonor THEN
	 PRINT #3, USING "GEOGR.S.             ####        ####"; DECLg; INCLg
 END IF
 IF Tectcorr > 0 AND (IrhoA <> 0 OR Imu <> 0 OR ItauA <> 0 OR IsigA <> 0) THEN
	 PRINT #3, USING "TILT Corr.           ####        ####"; DECLpt; INCLpt
 END IF
 IF Tectcorr = 2 AND (ItauA <> 0 OR IsigA <> 0) THEN
	 PRINT #3, USING "FULL Corr.           ####        ####"; DECLpf; INCLpf
 END IF
 PRINT #3, "-------------------------------------------"
RETURN


'***** print screen to file .TXT
printscreen:

 PRINT #3, CHR$(1)
 GOSUB printheader
 GOSUB printdata
 GOSUB printaver
 GOSUB printresults
 PRINT #3, ""
 PRINT #3, ""
RETURN


'***** evaluation of measured data
eval:

GOSUB testmode
SELECT CASE Mode
	CASE 0
		 RETURN
	CASE 2
		x = T(1)
		y = T(3)
		z = (T(2) + T(4)) / 2
		Prec = ABS(T(2) - T(4)) / 2
		Alfa = 999
		IF mmode > 0 THEN RETURN
	CASE 3
		x = (T(1) + T(5)) / 2
		Xo = x
		y = (T(3) + T(6)) / 2
		Yo = y
		z = (T(2) + T(4)) / 2
		IF Stype = 1 THEN
			x = (Xo + Yo) * .707
			y = (-Xo + Yo) * .707
		END IF
		T = T(1) - Xo: w = T * T
		T = T(2) - z: w = w + T * T
		T = T(3) - Yo: w = w + T * T
		T = T(4) - z: w = w + T * T
		T = T(5) - Xo: w = w + T * T
		T = T(6) - Yo: w = w + T * T
		Prec = SQR(w / 6)
		CALL Uhel(T(1), T(6), T(4), T(5), T(3), T(2), Alfa)
	CASE 4
		x = (T(1) + T(5)) / 2
		y = (T(3) + T(7)) / 2
		z = (T(2) + T(4) + T(6) + T(8)) / 4
		P1 = (T(1) + T(3) - T(5) - T(7)) / 4
		Q1 = (T(2) + T(4) - T(6) - T(8)) / 4
	CASE 6
		x = (T(1) + T(5) + T(9) + T(11)) / 4
		y = (T(3) + T(7) + T(10) + T(12)) / 4
		z = (T(2) + T(4) + T(6) + T(8)) / 4
		P1 = (T(1) + T(3) - T(5) - T(7) - T(9) + T(11)) / 6
		Q1 = (T(2) + T(4) - T(6) - T(8) - T(10) + T(12)) / 6
END SELECT
IF Mode > 3 THEN
	T = T(1) - x - P1: w = T * T
	Top(1) = T(1) - P1
	T = T(2) - z - Q1: w = w + T * T
	Top(2) = T(2) - Q1
	T = T(3) - y - P1: w = w + T * T
	Top(3) = T(3) - P1
	T = T(4) - z - Q1: w = w + T * T
	Top(4) = T(4) - Q1
	T = T(5) - x + P1: w = w + T * T
	Top(5) = T(5) + P1
	T = T(6) - z + Q1: w = w + T * T
	Top(6) = T(6) + Q1
	T = T(7) - y + P1: w = w + T * T
	Top(7) = T(7) + P1
	T = T(8) - z + Q1: w = w + T * T
	Top(8) = T(8) + Q1
	Prec = SQR(w / 6)
	CALL Uhel(Top(1), Top(3), (Top(6) + Top(8)) / 2, Top(5), Top(7), (Top(2) + Top(4)) / 2, Alfa)
	IF Mode = 4 THEN GOTO evaln
	T = T(9) - x + P1: w = w + T * T
	Top(9) = T(9) + P1
	T = T(10) - y + Q1: w = w + T * T
	Top(10) = T(10) + Q1
	T = T(11) - x - P1: w = w + T * T
	Top(11) = T(11) - P1
	T = T(12) - y - Q1: w = w + T * T
	Top(12) = T(12) - Q1
	Prec = SQR(w / 28)
END IF
evaln:
	Ampl = SQR(x * x + y * y + z * z)
	Prec = 100 * Prec / (Ampl + 1E-12)
	IF Mode = 2 AND T(5) <> 9999 THEN
		KEY 8, "      "
		KEY(8) OFF
		RETURN
	END IF
	IF Mode = 4 AND T(9) <> 9999 THEN
		KEY 8, "      "
		KEY(8) OFF
		Top(9) = T(9) + P1
		Top(10) = T(10) + Q1
		retf = 0
		GOSUB dispdata
		RETURN
	END IF
	retf = 7
RETURN


testmode:
 Mode = 0
 FOR i = 1 TO 12
	 IF T(i) = 9999 THEN GOTO kf
 NEXT i
kf:
IF Htype < 2 THEN                ' auto and semi-auto holder
	IF i > 6 THEN Mode = 3
ELSE
	IF i > 4 THEN Mode = 2        ' manual holder
	IF i > 8 THEN Mode = 4
	IF i > 12 THEN Mode = 6
END IF
RETURN


'*****     Arc tangent of two arguments - result in deg
arctg2:

IF xx = 0 THEN
	atn2 = SGN(yy) * pi / 2
ELSE
	atn2 = ATN(yy / xx) + pi * (xx < 0) * (1 + 2 * (yy >= 0))
END IF
atn2 = atn2 * deg
RETURN


'***** compute Declination and Inclination (XV,YV,ZV,  DECL,INCL)

DeclIncl:
 yy = YV
 xx = XV
 GOSUB arctg2
 Decl = atn2
 IF Decl < 0 THEN Decl = Decl + 360
 yy = ZV
 xx = SQR(XV * XV + YV * YV):
 GOSUB arctg2
 Incl = atn2
RETURN


'***** transformation to paleogeografic system

paleo:

 GOSUB Rotpl
 b(1, 1) = Xg: b(2, 1) = Yg: b(3, 1) = Zg
 FOR i = 1 TO 3: FOR j = 1 TO 3: a(i, j) = R(i, j): NEXT j: NEXT i
 L = 3: m = 3: N = 1
 GOSUB Mxmp11
 XP = C(1, 1): YP = C(2, 1): ZP = C(3, 1)
RETURN


'***** Matrix multiplication   |c| = |a| * |b|
Mxmp11:

 FOR i = 1 TO L
	 FOR k = 1 TO N
		s = 0
		FOR j = 1 TO m
		  s = s + a(i, j) * b(j, k)
		NEXT j
		C(i, k) = s
	 NEXT k
 NEXT i
RETURN


'***** compute rotation matrix
Rotar:

 T = OMEGA / deg
 s = SIN(T)
 c1 = 1 - COS(T)
 FOR i = 1 TO 3
	j = i MOD 3
	j = j + 1
	k = j MOD 3
	k = k + 1
	R(i, k) = d(i) * d(k) * c1 + d(j) * s
	R(j, k) = d(j) * d(k) * c1 - d(i) * s
	R(k, k) = 1 - (1 - d(k) * d(k)) * c1
 NEXT i
RETURN


'***** simple or full tectonic correction
Rotpl:

 T = Irho / deg
 d(1) = -SIN(T): d(2) = COS(T): d(3) = 0
 OMEGA = Imu
 GOSUB Rotar                                     ' to horizontal plane
 IF Isig = 0 THEN RETURN                         ' simple corr.
 FOR i = 1 TO 3: FOR j = 1 TO 3                  ' full corr.
	  a(i, j) = R(i, j): R2(i, j) = R(i, j)
 NEXT j: NEXT i
 T = Itau / deg: COST = COS(T): SINT = SIN(T)
 T = Isig / deg: COSS = COS(T): SINS = SIN(T)
 b(1, 1) = COST * COSS: b(2, 1) = SINT * COSS: b(3, 1) = SINS
 L = 3: m = 3: N = 1
 GOSUB Mxmp11
 yy = C(2, 1): xx = C(1, 1)
 GOSUB arctg2
 OMEGA = Itau - atn2
 d(1) = 0: d(2) = 0: d(3) = 1
 GOSUB Rotar                                     ' rotation about vert.axis
 FOR i = 1 TO 3: FOR j = 1 TO 3
	  a(i, j) = R(i, j): b(i, j) = R2(i, j)
 NEXT j: NEXT i
 N = 3
 GOSUB Mxmp11                                    ' result matrix
 FOR i = 1 TO 3: FOR j = 1 TO 3
	  R(i, j) = C(i, j)
 NEXT j: NEXT i
RETURN


'------------------------------------------------------------


'***** Three-way decision of entering data
selway:

GOSUB keysoff
PRINT
PRINT "--> Select way how to input data : ?  "
PRINT "    Input from geological file     F1  "
PRINT "    Manual input from memo-book    F2  "
PRINT "    Non-oriented specimen          F3  "
LOCATE CSRLIN - 4, 36
zpet19:
DO
w$ = INKEY$
LOOP WHILE w$ = ""
sel = ASC(RIGHT$(w$, 1))
IF sel = 59 OR sel = 60 OR sel = 61 THEN
	entermode = sel - 58
ELSE
	BEEP
	GOTO zpet19
END IF
COLOR 15
PRINT "F" + RIGHT$(STR$(entermode), 1)
COLOR 7
LOCATE CSRLIN + 4, 1
Pause .5

SELECT CASE entermode
CASE 1
	geopath$ = path$

testgeo:
	CLS
	PRINT "*** Selection of the geological file "
	PRINT

	IF LEN(DIR$(geopath$ + "*.GED")) = 0 THEN
		BEEP
		PRINT "### No Geological File in "; geopath$
		PRINT

ingeo:
		PRINT "The current path is  "; CURDIR$
		PRINT "--> Press <CR> to confirm or Enter a new path "
		PRINT
		PRINT "Enter Path of the Geological file (drive:\dir1\dir2\...) ";
		INPUT geopath$
		geopath$ = UCASE$(geopath$)
		PRINT
		ilinz = CSRLIN
		IF geopath$ = "" THEN geopath$ = CURDIR$
		IF LEN(geopath$) = 1 THEN geopath$ = geopath$ + ":"
		IF RIGHT$(geopath$, 1) <> "\" THEN geopath$ = geopath$ + "\"
		ON ERROR GOTO invalidgeopath
		ilinz = CSRLIN
		FILES geopath$ + "*.GED"
		ON ERROR GOTO 0
		ilink = CSRLIN
		LOCATE ilinz, 1
		PRINT SPACE$(79);
		LOCATE ilinz, 1
		PRINT "Directory "; geopath$
		LOCATE ilink - 1, 1
		PRINT SPACE$(79);
		GOTO testgeo
	ELSE
		ilinz = CSRLIN
		FILES geopath$ + "*.GED"
		ilink = CSRLIN
		LOCATE ilinz, 1
		PRINT SPACE$(79);
		LOCATE ilinz, 1
		PRINT "Directory "; geopath$
		LOCATE ilink - 1, 1
		PRINT SPACE$(79);
	END IF

	PRINT
	PRINT "*** To change path enter #"
	INPUT "--> Enter name of .GED data file (max.8 chars) "; ownname$
	IF ownname$ = "#" THEN
		CLS
		PRINT "*** Selection of the geological file "
		PRINT
		GOTO ingeo
	END IF
	Filegeo$ = geopath$ + ownname$ + ".GED"
	
	CLOSE (10)
	OPEN Filegeo$ FOR RANDOM AS #10 LEN = 64
	IF LOF(10) = 0 THEN
		CLOSE (10)
		KILL Filegeo$
		PRINT
		PRINT "### Geological File "; Filegeo$; " not found"
		BEEP
		PRINT
		PRINT "--> Press any key to continue "
		q$ = INPUT$(1)
		GOTO testgeo
	ELSE
		GET #10, 1, geol1
		he.nameloc = geol1.namloc
		he.longi = geol1.long
		he.lati = geol1.lat
		he.rockt = geol1.rock
		he.strati = geol1.strat
		he.litos = geol1.lito
		he.regio = geol1.regi
		prg(1) = VAL(geol1.pr1)
		prg(2) = VAL(geol1.pr2)
		prg(3) = VAL(geol1.pr3)
		prg(4) = VAL(geol1.pr4)
	END IF
	FOR i = 1 TO 4
	  IF z(i) <> prg(i) THEN
		  PRINT
		  PRINT "Warning :"
		  PRINT "### Orientation parameters in Geological file and"
		  PRINT "    in Configuration file are different"
		  PRINT
		  BEEP
		  PRINT Filegeo$; TAB(55); USING " ###"; prg(1); prg(2); prg(3); prg(4)
		  PRINT
		  PRINT path$ + "PARJR6.CFG"; TAB(55); USING " ###"; z(1); z(2); z(3); z(4)
		  PRINT
		  FOR op = 1 TO 4
			  z(op) = prg(op)
		  NEXT op
		  GOSUB testorparam
		  IF NOT badopf THEN
			  PRINT "*** Orientation parameters will be reset according"
			  PRINT "    the Geological file which has higher priority"
			  PRINT
			  PRINT "--> Confirm or Exit the program [C/E] ?"
zpet11:         q$ = INPUT$(1)
			  IF UCASE$(q$) = "C" THEN
				  GOSUB writeconfig
				  geoldata = -1
				  RETURN
			  ELSEIF UCASE$(q$) = "E" THEN
				  GOTO mk10a
			  ELSE
				  BEEP
				  GOTO zpet11
			  END IF
		  ELSE
			  PRINT "### Orientation parameters in Geological file are illegal"
			  PRINT
			  PRINT "--> Press any key to exit program"
			  q$ = INPUT$(1)
			  GOTO mk10a
		  END IF
	  END IF
	NEXT i
	geoldata = -1
CASE 3
	nonor = -1
END SELECT
RETURN


'***** search geol. data in .GED file

searchgd:
	ngedrec = LOF(10) \ 64
	FOR ir = 2 TO ngedrec
		GET #10, ir, geol
		namg$ = UCASE$(geol.nam)
		namg$ = UCASE$(RTRIM$(namg$))
		m = INSTR(namk$, namg$)
		IF m > 0 THEN
			EXIT FOR
		END IF
	NEXT ir
	IF ir = ngedrec + 1 THEN
		PRINT
		PRINT "### Record not found";
		PRINT
		BEEP
		findf = 0
		Tectcorr = 2
	ELSE
		PRINT
		PRINT "*** Record found :";
		PRINT
		findf = -1
		Tectcorr = 0
		IrhoA = 0: : Irho = 0: Imu = 0: ItauA = 0: Itau = 0: IsigA = 0: Isig = 0

		the = VAL(geol.azim)              'retrieving the tectonic parameters
		psi = VAL(geol.plun)

		IF the = 0 AND psi = 0 THEN the = 1

		IpfiA = the: Ipfi = the                '--------------
		IpsiA = psi: Ipsi = psi                '---------------


		bfs(1, 1) = UCASE$(geol.bfs11)
		bfs(2, 1) = UCASE$(geol.bfs21)
		rho(1) = VAL(geol.dfoli1)
		amu(1) = VAL(geol.ifoli1)
		tau(1) = VAL(geol.dline1)
		sig(1) = VAL(geol.iline1)

		bfs(1, 2) = UCASE$(geol.bfs12)
		bfs(2, 2) = UCASE$(geol.bfs22)
		rho(2) = VAL(geol.dfoli2)
		amu(2) = VAL(geol.ifoli2)
		tau(2) = VAL(geol.dline2)
		sig(2) = VAL(geol.iline2)

		IF bfs(1, 1) = codefol$ THEN
			IrhoA = rho(1): Irho = rho(1)          '--------------
			Imu = amu(1)
			Tectcorr = 1
			IF bfs(2, 1) = codelin$ THEN
				ItauA = tau(1): Itau = tau(1)
				IsigA = sig(1): Isig = sig(1)
				Tectcorr = 2
				GOSUB disprecord
				RETURN
			ELSEIF bfs(2, 2) = codelin$ THEN
				ItauA = tau(2): Itau = tau(2)
				IsigA = sig(2): Isig = sig(2)
				Tectcorr = 2
				GOSUB disprecord
				RETURN
			END IF
		ELSEIF bfs(1, 2) = codefol$ THEN
			IrhoA = rho(2): Irho = rho(2)          '--------------
			Imu = amu(2)
			Tectcorr = 1
			IF bfs(2, 2) = codelin$ THEN
				ItauA = tau(2): Itau = tau(2)
				IsigA = sig(2): Isig = sig(2)
				Tectcorr = 2
				GOSUB disprecord
				RETURN
			ELSEIF bfs(2, 1) = codelin$ THEN
				ItauA = tau(1): Itau = tau(1)
				IsigA = sig(1): Isig = sig(1)
				Tectcorr = 2
				GOSUB disprecord
				RETURN
			END IF
		END IF
		IF Tectcorr < 2 THEN GOSUB disprecord
	END IF
RETURN


disprecord:
		PRINT
		PRINT USING "SPECIMEN ANGLES   ###  ###"; Ipfi; Ipsi
		PRINT

		PRINT USING "First Tecto   &&  ###  ###  ###  ###"; bfs(1, 1); bfs(2, 1); rho(1); amu(1); tau(1); sig(1)

		PRINT USING "Second Tecto  &&  ###  ###  ###  ###"; bfs(1, 2); bfs(2, 2); rho(2); amu(2); tau(2); sig(2)

		PRINT
		IF Tectcorr = 1 THEN
			PRINT USING "TECTONIC ANGLES   ###  ###"; Irho; Imu
		ELSEIF Tectcorr = 2 THEN
			PRINT USING "TECTONIC ANGLES   ###  ###  ###  ###"; Irho; Imu; Itau; Isig
		END IF
		PRINT
		PRINT "--> Press any key to continue"
		q$ = INPUT$(1)
RETURN


beep3x:
  BEEP: SLEEP 1: BEEP: SLEEP 1: BEEP
RETURN


'***** transformation of angles for matrix mode 6-0-6-0
coran:

 IF nonor THEN RETURN
 Ipfi = Ipfi + (IP1 - IP3) * 30
 Ipfi = Ipfi - (IP1 - 6) * 30
 IF IP2 = 90 THEN Ipsi = 90 - Ipsi
 F3 = (IP1 - 6) * 30
 F1 = Ipfi / deg
 F2 = Ipsi / deg
 F3 = F3 / deg
 Irho = IrhoA
 Irho = Irho + IP4
 IF Irho >= 360 THEN Irho = Irho - 360
RETURN

fictlin:
 IF Itau = 0 AND Isig = 0 AND Tectcorr > 0 THEN
	 Isig = 0
 END IF
RETURN

'  enable <ESC> key
enesc:
	FOR ii = 16 TO 19
	ON KEY(ii) GOSUB Esc
	KEY(ii) ON
	NEXT ii
RETURN

'  disable <ESC> key
diesc:
	FOR ii = 16 TO 19
	KEY(ii) OFF
	NEXT ii
RETURN

DEFINT I-N
'***************************************************************************
'
		 SUB Clearbuff
'
'           Purpose
'              Clearing input buffer of the serial chanell.
'              It is considered to be finished if in the last
'              0.1 s has not come any byte.
'
'***************************************************************************
start = TIMER
DO
	IF LOC(1) > 0 THEN
		ww$ = INPUT$(LOC(1), #1)
		start = TIMER
	END IF
	wt = TIMER - start
LOOP UNTIL LOC(1) = 0 AND wt > .1
END SUB

'===============================================================
'
	FUNCTION ENG$ (a, masize)
'
'  Purpose
'     To convert a number to engineering notation
'
'  Description of parameters
'     a        - given number
'     masize   - length of 'mantissa' (digits)
'     ENG$     - string of length  masize + 6
'
'  Procedures required
'     None
'
'==================================================================

aa = VAL(STR$(a))

IF aa <> 0 THEN
	isgn = SGN(aa)
	aa = ABS(aa)
	k = INT(LOG(aa) / LOG(10#))
	aa = aa * 10# ^ (-k + masize - 1)
	aa = VAL(STR$(aa))
	aa = INT(aa + .5)
	k2 = INT(k / 3) * 3
	k1 = k - k2
	aa = aa * 10# ^ (-masize + 1 + k1)
	IF aa >= 1000 THEN
		aa = aa / 1000
		k2 = k2 + 3
	END IF

	a$ = STR$(aa)
	le = LEN(a$)
	IF isgn = -1 THEN a$ = "-" + RIGHT$(a$, le - 1)
	i = 0: lov = 0

	DO
		i = i + 1
		IF MID$(a$, i, 1) = "." THEN lov = -1
	LOOP UNTIL lov OR i = le

	IF NOT lov THEN
		IF masize = 2 AND le = 4 THEN
			a$ = LEFT$(a$, 3) + "O"
		ELSE
			a$ = a$ + "."
		END IF
	END IF

	DO UNTIL LEN(a$) >= masize + 2
		a$ = a$ + "0"
	LOOP

	isgn = SGN(k2)
	b$ = STR$(k2)
	le = LEN(b$)
	b$ = RIGHT$(b$, le - 1)
	le = le - 1
	IF le = 1 THEN b$ = "0" + b$
	IF isgn = 1 OR isgn = 0 THEN
		b$ = "+" + b$
	ELSE
		b$ = "-" + b$
	END IF

ELSE
	a$ = " 0."
	DO
		a$ = a$ + "0"
	LOOP UNTIL LEN(a$) = masize + 2
	b$ = "+00"
END IF

ENG$ = a$ + "E" + b$
END FUNCTION

DEFSNG I-N
'=========================================================================
'
	SUB Pause (T)
'
'  Purpose
'     Wait for  t  seconds,  t  is real.
'
'  Procedures required
'     None
'
'=========================================================================

start = TIMER
DO
delay = TIMER - start
LOOP UNTIL delay > T
END SUB

SUB Uhel (Mx1, My1, Mz1, Mx2, My2, Mz2, phi)

	M1 = SQR(Mx1 * Mx1 + My1 * My1 + Mz1 * Mz1)
	M2 = SQR(Mx2 * Mx2 + My2 * My2 + Mz2 * Mz2)
	IF M1 = 0 THEN M1 = M1 + 1E-15
	IF M2 = 0 THEN M2 = M2 + 1E-15
	a1 = Mx1 / M1: a2 = My1 / M1: a3 = Mz1 / M1
	b1 = Mx2 / M2: b2 = My2 / M2: b3 = Mz2 / M2
	aa = SQR(a1 * a1 + a2 * a2 + a3 * a3)
	bb = SQR(b1 * b1 + b2 * b2 + b3 * b3)
	aabb = aa * bb
	IF aabb = 0 THEN aabb = 1E-15
	cosphi = (a1 * b1 + a2 * b2 + a3 * b3) / aabb
	IF cosphi = 0 THEN cosphi = 1E-15
	phi = ATN(SQR(ABS(1 - cosphi * cosphi)) / cosphi)

END SUB

